@@ -9,11 +9,15 @@ Cu.import("resource:///modules/imServices.jsm");
 Cu.import("resource:///modules/ircUtils.jsm");
 Cu.import("resource:///modules/ircHandlers.jsm");
 Cu.import("resource:///modules/jsProtoHelper.jsm");
+Cu.import("resource:///modules/NormalizedMap.jsm");
 Cu.import("resource:///modules/socket.jsm");
 
 XPCOMUtils.defineLazyModuleGetter(this, "PluralForm",
   "resource://gre/modules/PluralForm.jsm");
 
+XPCOMUtils.defineLazyModuleGetter(this, "DownloadUtils",
+  "resource://gre/modules/DownloadUtils.jsm");
+
 /*
  * Parses a raw IRC message into an object (see section 2.3 of RFC 2812). This
  * returns an object with the following fields:
@@ -90,13 +94,13 @@ function ircMessage(aData) {
 function _setMode(aAddNewMode, aNewModes) {
   // Check each mode being added/removed.
   for each (let newMode in aNewModes) {
-    let index = this._modes.indexOf(newMode);
+    let hasMode = this._modes.has(newMode);
     // If the mode is in the list of modes and we want to remove it.
-    if (index != -1 && !aAddNewMode)
-      this._modes.splice(index, 1);
+    if (hasMode && !aAddNewMode)
+      this._modes.delete(newMode);
     // If the mode is not in the list of modes and we want to add it.
-    else if (index == -1 && aAddNewMode)
-      this._modes.push(newMode);
+    else if (!hasMode && aAddNewMode)
+      this._modes.add(newMode);
   }
 }
 
@@ -209,8 +213,8 @@ const GenericIRCConversation = {
     // If we are waiting for the conversation name, set it.
     let account = this._account;
     if (this._waitingForNick && nick == this.normalizedName) {
-      if (hasOwnProperty(account.whoisInformation, nick))
-        this.updateNick(account.whoisInformation[nick]["nick"]);
+      if (account.whoisInformation.has(nick))
+        this.updateNick(account.whoisInformation.get(nick)["nick"]);
       delete this._waitingForNick;
       return;
     }
@@ -218,15 +222,15 @@ const GenericIRCConversation = {
     // Otherwise, print the requested whois information.
     let type = {system: true, noLog: true};
     // RFC 2812 errors 401 and 406 result in there being no entry for the nick.
-    if (!hasOwnProperty(account.whoisInformation, nick)) {
+    if (!account.whoisInformation.has(nick)) {
       this.writeMessage(null, _("message.unknownNick", nick), type);
       return;
     }
     // If the nick is offline, tell the user. In that case, it's WHOWAS info.
     let msgType = "message.whois";
-    if ("offline" in account.whoisInformation[nick])
+    if ("offline" in account.whoisInformation.get(nick))
       msgType = "message.whowas";
-    let msg = _(msgType, account.whoisInformation[nick]["nick"]);
+    let msg = _(msgType, account.whoisInformation.get(nick)["nick"]);
 
     // Iterate over each field.
     let tooltipInfo = aSubject.QueryInterface(Ci.nsISimpleEnumerator);
@@ -239,6 +243,12 @@ const GenericIRCConversation = {
           break;
         case Ci.prplITooltipInfo.sectionBreak:
           break;
+        case Ci.prplITooltipInfo.status:
+          if (elt.label != Ci.imIStatusInfo.STATUS_AWAY)
+            break;
+          // The away message has no tooltipInfo.pair entry.
+          msg += "\n" + _("message.whoisEntry", _("tooltip.away"), elt.value);
+          break;
       }
     }
     this.writeMessage(null, msg, type);
@@ -253,14 +263,15 @@ const GenericIRCConversation = {
 
 function ircChannel(aAccount, aName, aNick) {
   this._init(aAccount, aName, aNick);
-  this._modes = [];
+  this._participants = new NormalizedMap(this.normalizeNick.bind(this));
+  this._modes = new Set();
   this._observedNicks = [];
   this.banMasks = [];
   this._firstJoin = true;
 }
 ircChannel.prototype = {
   __proto__: GenericConvChatPrototype,
-  _modes: [],
+  _modes: null,
   _receivedInitialMode: false,
   // For IRC you're not in a channel until the JOIN command is received, open
   // all channels (initially) as left.
@@ -313,16 +324,12 @@ ircChannel.prototype = {
   // Use the normalized nick in order to properly notify the observers.
   getNormalizedChatBuddyName: function(aNick) this.normalizeNick(aNick),
 
-  hasParticipant: function(aNick)
-    hasOwnProperty(this._participants, this.normalizeNick(aNick)),
-
   getParticipant: function(aNick, aNotifyObservers) {
-    let normalizedNick = this.normalizeNick(aNick);
-    if (this.hasParticipant(aNick))
-      return this._participants[normalizedNick];
+    if (this._participants.has(aNick))
+      return this._participants.get(aNick);
 
     let participant = new ircParticipant(aNick, this);
-    this._participants[normalizedNick] = participant;
+    this._participants.set(aNick, participant);
 
     // Add the participant to the whois table if it is not already there.
     this._account.setWhois(participant._name);
@@ -334,7 +341,7 @@ ircChannel.prototype = {
     return participant;
   },
   updateNick: function(aOldNick, aNewNick) {
-    let isParticipant = this.hasParticipant(aOldNick);
+    let isParticipant = this._participants.has(aOldNick);
     if (this.normalizeNick(aOldNick) == this.normalizeNick(this.nick)) {
       // If this is the user's nick, change it.
       this.nick = aNewNick;
@@ -350,43 +357,44 @@ ircChannel.prototype = {
 
     // Get the original ircParticipant and then remove it.
     let participant = this.getParticipant(aOldNick);
-    this.removeParticipant(aOldNick);
+    this._participants.delete(aOldNick);
 
     // Update the nickname and add it under the new nick.
     participant._name = aNewNick;
-    this._participants[this.normalizeNick(aNewNick)] = participant;
+    this._participants.set(aNewNick, participant);
 
     this.notifyObservers(participant, "chat-buddy-update", aOldNick);
   },
-  removeParticipant: function(aNick, aNotifyObservers) {
-    if (!this.hasParticipant(aNick))
+  removeParticipant: function(aNick) {
+    if (!this._participants.has(aNick))
       return;
 
-    if (aNotifyObservers) {
-      let stringNickname = Cc["@mozilla.org/supports-string;1"]
-                              .createInstance(Ci.nsISupportsString);
-      stringNickname.data = aNick;
-      this.notifyObservers(new nsSimpleEnumerator([stringNickname]),
-                           "chat-buddy-remove");
-    }
-    delete this._participants[this.normalizeNick(aNick)];
+    let stringNickname = Cc["@mozilla.org/supports-string;1"]
+                           .createInstance(Ci.nsISupportsString);
+    stringNickname.data = aNick;
+    this.notifyObservers(new nsSimpleEnumerator([stringNickname]),
+                         "chat-buddy-remove");
+    this._participants.delete(aNick);
   },
   // Use this before joining to avoid errors of trying to re-add an existing
   // participant
   removeAllParticipants: function() {
     let stringNicknames = [];
-    for (let nickname in this._participants) {
+    this._participants.forEach(function(aParticipant) {
       let stringNickname = Cc["@mozilla.org/supports-string;1"]
                               .createInstance(Ci.nsISupportsString);
-      stringNickname.data = this._participants[nickname].name;
+      stringNickname.data = aParticipant.name;
       stringNicknames.push(stringNickname);
-    }
+    });
     this.notifyObservers(new nsSimpleEnumerator(stringNicknames),
                          "chat-buddy-remove");
-    this._participants = {};
+    this._participants.clear();
   },
 
   setMode: function(aNewMode, aModeParams, aSetter) {
+    // Save this for a comparison after the new modes have been set.
+    let previousTopicSettable = this.topicSettable;
+
     const hostMaskExp = /^.+!.+@.+$/;
     function getNextParam() {
       // If there's no next parameter, throw a warning.
@@ -412,7 +420,7 @@ ircChannel.prototype = {
 
     // Check each mode being added and update the user.
     let channelModes = [];
-    let userModes = {};
+    let userModes = new NormalizedMap(this.normalizeNick.bind(this));
     let msg;
 
     for (let i = aNewMode.length - 1; i > 0; --i) {
@@ -420,13 +428,13 @@ ircChannel.prototype = {
       // implementations, check if a participant with that name exists. If this
       // is true, then update the mode of the ConvChatBuddy.
       if (this._account.memberStatuses.indexOf(aNewMode[i]) != -1 &&
-          aModeParams.length && this.hasParticipant(peekNextParam())) {
+          aModeParams.length && this._participants.has(peekNextParam())) {
         // Store the new modes for this nick (so each participant's mode is only
         // updated once).
-        let nick = this.normalizeNick(getNextParam());
-        if (!hasOwnProperty(userModes, nick))
-          userModes[nick] = [];
-        userModes[nick].push(aNewMode[i]);
+        let nick = getNextParam();
+        if (!userModes.has(nick))
+          userModes.set(nick, []);
+        userModes.get(nick).push(aNewMode[i]);
 
         // Don't use this mode as a channel mode.
         continue;
@@ -489,8 +497,8 @@ ircChannel.prototype = {
       this.WARN("Unused mode parameters: " + aModeParams.join(", "));
 
     // Update the mode of each participant.
-    for (let nick in userModes)
-      this.getParticipant(nick).setMode(addNewMode, userModes[nick], aSetter);
+    for (let [nick, mode] of userModes.entries())
+      this.getParticipant(nick).setMode(addNewMode, mode, aSetter);
 
     if (!channelModes.length)
       return;
@@ -502,48 +510,45 @@ ircChannel.prototype = {
     msg = _("message.channelmode", aNewMode[0] + channelModes.join(""),
             aSetter);
     this.writeMessage(aSetter, msg, {system: true});
-    this.checkTopicSettable();
+
+    // If the topic can now be set (and it couldn't previously) or vice versa,
+    // notify the UI.
+    if (this.topicSettable != previousTopicSettable)
+      this.notifyObservers(this, "chat-update-topic");
 
     this._receivedInitialMode = true;
   },
 
   setModesFromRestriction: function(aRestriction) {
     // First remove all types from the list of modes.
-    for each (let mode in this._account.channelRestrictionToModeMap) {
-      let index = this._modes.indexOf(mode);
-      this._modes.splice(index, index != -1);
-    }
+    for each (let mode in this._account.channelRestrictionToModeMap)
+      this._modes.delete(mode);
 
     // Add the new mode onto the list.
     if (aRestriction in this._account.channelRestrictionToModeMap) {
       let mode = this._account.channelRestrictionToModeMap[aRestriction];
       if (mode)
-        this._modes.push(mode);
+        this._modes.add(mode);
     }
   },
 
   get topic() this._topic, // can't add a setter without redefining the getter
   set topic(aTopic) {
+    // Note that the UI isn't updated here because the server will echo back the
+    // TOPIC to us and we'll set it on receive.
     this._account.sendMessage("TOPIC", [this.name, aTopic]);
   },
-  _previousTopicSettable: null,
-  checkTopicSettable: function() {
-    if (this.topicSettable == this._previousTopicSettable &&
-        this._previousTopicSettable != null)
-      return;
-
-    this.notifyObservers(this, "chat-update-topic");
-  },
   get topicSettable() {
-    // If we're not in the room yet, we don't exist.
-    if (!this.hasParticipant(this.nick))
+    // Don't use getParticipant since we don't want to lazily create it!
+    let participant = this._participants.get(this.nick);
+
+    // We must be in the room to set the topic.
+    if (!participant)
       return false;
 
     // If the channel mode is +t, hops and ops can set the topic; otherwise
     // everyone can.
-    let participant = this.getParticipant(this.nick);
-    return this._modes.indexOf("t") == -1 || participant.op ||
-           participant.halfOp;
+    return !this._modes.has("t") || participant.op || participant.halfOp;
   }
 };
 copySharedBaseToPrototype(GenericIRCConversation, ircChannel.prototype);
@@ -552,13 +557,16 @@ function ircParticipant(aName, aConv) {
   this._name = aName;
   this._conv = aConv;
   this._account = aConv._account;
-  this._modes = [];
+  this._modes = new Set();
 
   // Handle multi-prefix modes.
   let i;
   for (i = 0; i < this._name.length &&
-              this._name[i] in this._account.userPrefixToModeMap; ++i)
-    this._modes.push(this._account.userPrefixToModeMap[this._name[i]]);
+              this._name[i] in this._account.userPrefixToModeMap; ++i) {
+    let mode = this._account.userPrefixToModeMap[this._name[i]];
+    if (mode)
+      this._modes.add(mode);
+  }
   this._name = this._name.slice(i);
 }
 ircParticipant.prototype = {
@@ -572,26 +580,19 @@ ircParticipant.prototype = {
                 this.name, aSetter);
     this._conv.writeMessage(aSetter, msg, {system: true});
     this._conv.notifyObservers(this, "chat-buddy-update");
-
-    // In case the new mode now lets us edit the topic.
-    if (this._account.normalize(this.name) ==
-        this._account.normalize(this._account._nickname))
-      this._conv.checkTopicSettable();
   },
 
-  get voiced() this._modes.indexOf("v") != -1,
-  get halfOp() this._modes.indexOf("h") != -1,
-  get op() this._modes.indexOf("o") != -1,
-  get founder()
-    this._modes.indexOf("O") != -1 || this._modes.indexOf("q") != -1,
+  get voiced() this._modes.has("v"),
+  get halfOp() this._modes.has("h"),
+  get op() this._modes.has("o"),
+  get founder() this._modes.has("O") || this._modes.has("q"),
   get typing() false
 };
 
 function ircConversation(aAccount, aName) {
-  this.buddy = aAccount.getBuddy(aName);
   let nick = aAccount.normalize(aName);
-  if (hasOwnProperty(aAccount.whoisInformation, nick))
-    aName = aAccount.whoisInformation[nick]["nick"];
+  if (aAccount.whoisInformation.has(nick))
+    aName = aAccount.whoisInformation.get(nick)["nick"];
 
   this._init(aAccount, aName);
   this._observedNicks = [];
@@ -603,6 +604,7 @@ function ircConversation(aAccount, aName) {
 }
 ircConversation.prototype = {
   __proto__: GenericConvIMPrototype,
+  get buddy() this._account.buddies.get(this.name),
 
   // Overwrite the writeMessage function to apply CTCP formatting before
   // display.
@@ -775,9 +777,9 @@ ircAccountBuddy.prototype = {
 };
 
 function ircAccount(aProtocol, aImAccount) {
-  this._buddies = {};
   this._init(aProtocol, aImAccount);
-  this._conversations = {};
+  this.buddies = new NormalizedMap(this.normalizeNick.bind(this));
+  this.conversations = new NormalizedMap(this.normalize.bind(this));
 
   // Split the account name into usable parts.
   let splitter = this.name.lastIndexOf("@");
@@ -790,8 +792,7 @@ function ircAccount(aProtocol, aImAccount) {
   // For more information, see where these are defined in the prototype below.
   this.trackQueue = [];
   this.pendingIsOnQueue = [];
-  this.whoisInformation = {};
-  this._chatRoomFieldsList = {};
+  this.whoisInformation = new NormalizedMap(this.normalizeNick.bind(this));
   this._caps = [];
 
   this._roomInfoCallbacks = new Set();
@@ -892,7 +893,7 @@ ircAccount.prototype = {
   },
 
   // The user's user mode.
-  _modes: [],
+  _modes: null,
   _userModeReceived: false,
   setUserMode: function(aNick, aNewModes, aSetter, aDisplayFullMode) {
     if (this.normalizeNick(aNick) != this.normalizeNick(this._nickname)) {
@@ -917,7 +918,7 @@ ircAccount.prototype = {
     if (this._showServerTab) {
       let msg;
       if (aDisplayFullMode)
-        msg = _("message.yourmode", this._modes.join(""));
+        msg = _("message.yourmode", [mode for (mode of this._modes)].join(""));
       else {
         msg = _("message.usermode", aNewModes, aNick,
                 aSetter || this._currentServerName);
@@ -979,7 +980,7 @@ ircAccount.prototype = {
 
   // The whois information: nicks are used as keys and refer to a map of field
   // to value.
-  whoisInformation: {},
+  whoisInformation: null,
   // Request WHOIS information on a buddy when the user requests more
   // information.
   requestBuddyInfo: function(aBuddyName) {
@@ -1001,11 +1002,10 @@ ircAccount.prototype = {
   },
   // Return an nsISimpleEnumerator of imITooltipInfo for a given nick.
   getBuddyInfo: function(aNick) {
-    let nick = this.normalizeNick(aNick);
-    if (!hasOwnProperty(this.whoisInformation, nick))
+    if (!this.whoisInformation.has(aNick))
       return EmptyEnumerator;
 
-    let whoisInformation = this.whoisInformation[nick];
+    let whoisInformation = this.whoisInformation.get(aNick);
     if (whoisInformation.serverName && whoisInformation.serverInfo) {
       whoisInformation.server =
         _("tooltip.serverValue", whoisInformation.serverName,
@@ -1025,6 +1025,16 @@ ircAccount.prototype = {
     // Convert booleans into a human-readable form.
     let normalizeBool = function(aBool) _(aBool ? "yes" : "no");
 
+    // Convert timespan in seconds into a human-readable form.
+    let normalizeTime = function(aTime) {
+      let valuesAndUnits = DownloadUtils.convertTimeUnits(aTime);
+      // If the time is exact to the first set of units, trim off
+      // the subsequent zeroes.
+      if (!valuesAndUnits[2])
+        valuesAndUnits.splice(2, 2);
+      return _("tooltip.timespan", valuesAndUnits.join(" "));
+    };
+
     // List of the names of the info to actually show in the tooltip and
     // optionally a transform function to apply to the value. Each field here
     // maps to tooltip.<fieldname> in irc.properties.
@@ -1036,10 +1046,9 @@ ircAccount.prototype = {
       registered: normalizeBool,
       registeredAs: null,
       secure: normalizeBool,
-      away: null,
       ircOp: normalizeBool,
       bot: normalizeBool,
-      idleTime: null,
+      lastActivity: normalizeTime,
       channels: sortChannels
     };
 
@@ -1053,32 +1062,42 @@ ircAccount.prototype = {
       }
     }
 
+    const kSetIdleStatusAfterSeconds = 3600;
+    let statusType = Ci.imIStatusInfo.STATUS_AVAILABLE;
+    let statusText = "";
+    if ("away" in whoisInformation) {
+      statusType = Ci.imIStatusInfo.STATUS_AWAY;
+      statusText = whoisInformation["away"];
+    }
+    else if ("offline" in whoisInformation)
+      statusType = Ci.imIStatusInfo.STATUS_OFFLINE;
+    else if ("lastActivity" in whoisInformation &&
+             whoisInformation["lastActivity"] > kSetIdleStatusAfterSeconds)
+      statusType = Ci.imIStatusInfo.STATUS_IDLE;
+    tooltipInfo.push(new TooltipInfo(statusType, statusText, true));
+
     return new nsSimpleEnumerator(tooltipInfo);
   },
   // Remove a WHOIS entry.
-  removeBuddyInfo: function(aNick) {
-    let nick = this.normalizeNick(aNick);
-    if (hasOwnProperty(this.whoisInformation, nick))
-      delete this.whoisInformation[nick];
-  },
+  removeBuddyInfo: function(aNick) this.whoisInformation.delete(aNick),
   // Copies the fields of aFields into the whois table. If the field already
   // exists, that field is ignored (it is assumed that the first server response
   // is the most up to date information, as is the case for 312/314). Note that
   // the whois info for a nick is reset whenever whois information is requested,
   // so the first response from each whois is recorded.
   setWhois: function(aNick, aFields = {}) {
-    let nick = this.normalizeNick(aNick);
     // If the nickname isn't in the list yet, add it.
-    if (!hasOwnProperty(this.whoisInformation, nick))
-      this.whoisInformation[nick] = {};
+    if (!this.whoisInformation.has(aNick))
+      this.whoisInformation.set(aNick, {});
 
     // Set non-normalized nickname field.
-    this.whoisInformation[nick]["nick"] = aNick;
+    let whoisInfo = this.whoisInformation.get(aNick);
+    whoisInfo.nick = aNick;
 
     // Set the WHOIS fields, but only the first time a field is set.
     for (let field in aFields) {
-      if (!this.whoisInformation[nick].hasOwnProperty(field))
-        this.whoisInformation[nick][field] = aFields[field];
+      if (!whoisInfo.hasOwnProperty(field))
+        whoisInfo[field] = aFields[field];
     }
 
     return true;
@@ -1098,60 +1117,46 @@ ircAccount.prototype = {
   },
   addBuddy: function(aTag, aName) {
     let buddy = new ircAccountBuddy(this, null, aTag, aName);
-    this._buddies[buddy.normalizedName] = buddy;
+    this.buddies.set(buddy.normalizedName, buddy);
     this.trackBuddy(buddy.userName);
 
     Services.contacts.accountBuddyAdded(buddy);
   },
   removeBuddy: function(aBuddy) {
-    delete this._buddies[aBuddy.normalizedName];
+    this.buddies.delete(aBuddy.normalizedName);
     this.untrackBuddy(aBuddy.userName);
   },
   // Loads a buddy from the local storage. Called for each buddy locally stored
   // before connecting to the server.
   loadBuddy: function(aBuddy, aTag) {
     let buddy = new ircAccountBuddy(this, aBuddy, aTag);
-    this._buddies[buddy.normalizedName] = buddy;
+    this.buddies.set(buddy.normalizedName, buddy);
     this.trackBuddy(buddy.userName);
 
     return buddy;
   },
-  hasBuddy: function(aName)
-    hasOwnProperty(this._buddies, this.normalizeNick(aName)),
-  // Return an array of buddy names.
-  getBuddyNames: function() {
-    let buddies = [];
-    for each (let buddyName in Object.keys(this._buddies))
-      buddies.push(this._buddies[buddyName].userName);
-    return buddies;
-  },
-  getBuddy: function(aName) {
-    if (this.hasBuddy(aName))
-      return this._buddies[this.normalizeNick(aName)];
-    return null;
-  },
   changeBuddyNick: function(aOldNick, aNewNick) {
     let msg;
     if (this.normalizeNick(aOldNick) == this.normalizeNick(this._nickname)) {
       // Your nickname changed!
       this._nickname = aNewNick;
       msg = _("message.nick.you", aNewNick);
-      for each (let conversation in this._conversations) {
+      this.conversations.forEach(conversation => {
         // Update the nick for chats, and inform the user in every conversation.
         if (conversation.isChat)
           conversation.updateNick(aOldNick, aNewNick);
         conversation.writeMessage(aOldNick, msg, {system: true});
-      }
+      });
     }
     else {
       msg = _("message.nick", aOldNick, aNewNick);
-      for each (let conversation in this._conversations) {
-        if (conversation.isChat && conversation.hasParticipant(aOldNick)) {
+      this.conversations.forEach(conversation => {
+        if (conversation.isChat && conversation._participants.has(aOldNick)) {
           // Update the nick in every chat conversation it is in.
           conversation.updateNick(aOldNick, aNewNick);
           conversation.writeMessage(aOldNick, msg, {system: true});
         }
-      }
+      });
     }
 
     // Adjust the whois table where necessary.
@@ -1159,13 +1164,13 @@ ircAccount.prototype = {
     this.setWhois(aNewNick);
 
     // If a private conversation is open with that user, change its title.
-    if (this.hasConversation(aOldNick)) {
+    if (this.conversations.has(aOldNick)) {
       // Get the current conversation and rename it.
       let conversation = this.getConversation(aOldNick);
 
       // Remove the old reference to the conversation and create a new one.
       this.removeConversation(aOldNick);
-      this._conversations[this.normalizeNick(aNewNick)] = conversation;
+      this.conversations.set(aNewNick, conversation);
 
       conversation.updateNick(aNewNick);
       conversation.writeMessage(aOldNick, msg, {system: true});
@@ -1232,8 +1237,8 @@ ircAccount.prototype = {
       // the user attempted to change to a version of the nick with a lower or
       // absent number suffix, and this failed.
       let msg = _("message.nick.fail", this._nickname);
-      for each (let conversation in this._conversations)
-        conversation.writeMessage(this._nickname, msg, {system: true});
+      this.conversations.forEach(conversation =>
+        conversation.writeMessage(this._nickname, msg, {system: true}));
       return true;
     }
 
@@ -1420,17 +1425,14 @@ ircAccount.prototype = {
 
   createConversation: function(aName) this.getConversation(aName),
 
-  // Temporarily stores the prplIChatRoomFieldValues passed to joinChat for
-  // each channel to enable later reconnections.
-  _chatRoomFieldsList: {},
-
   // aComponents implements prplIChatRoomFieldValues.
   joinChat: function(aComponents) {
     let channel = aComponents.getValue("channel");
     if (!channel) {
       this.ERROR("joinChat called without a channel name.");
-      return;
+      return null;
     }
+
     // A channel prefix is required. If the user didn't include one,
     // we prepend # automatically to match the behavior of other
     // clients. Not doing it used to cause user confusion.
@@ -1438,21 +1440,26 @@ ircAccount.prototype = {
       channel = "#" + channel;
 
     // No need to join a channel we are already in.
-    if (this.hasConversation(channel) && !this.getConversation(channel).left)
-      return;
+    if (this.conversations.has(channel)) {
+      let conv = this.getConversation(channel);
+      if (!conv.left)
+        return conv;
+    }
 
     let params = [channel];
     let key = aComponents.getValue("password");
     if (key)
       params.push(key);
     let defaultName = key ? channel + " " + key : channel;
-    this._chatRoomFieldsList[this.normalize(channel)] =
-      this.getChatRoomDefaultFieldValues(defaultName);
     // Send the join command, but don't log the channel key.
     this.sendMessage("JOIN", params,
                      "JOIN " + channel + (key ? " <key not logged>" : ""));
     // Open conversation early for better responsiveness.
-    this.getConversation(channel);
+    let conv = this.getConversation(channel);
+    // Store the prplIChatRoomFieldValues to enable later reconnections.
+    conv._chatRoomFields = this.getChatRoomDefaultFieldValues(defaultName);
+    conv.joining = true;
+    return conv;
   },
 
   chatRoomFields: {
@@ -1471,30 +1478,26 @@ ircAccount.prototype = {
   // Attributes
   get canJoinChat() true,
 
-  hasConversation: function(aConversationName)
-    hasOwnProperty(this._conversations, this.normalize(aConversationName)),
-
   // Returns a conversation (creates it if it doesn't exist)
   getConversation: function(aName) {
-    let name = this.normalize(aName);
-    // If the whois information has been received, we have the proper nick
-    // capitalization.
-    if (hasOwnProperty(this.whoisInformation, name))
-      aName = this.whoisInformation[name].nick;
-    if (!this.hasConversation(aName)) {
-      let constructor = this.isMUCName(aName) ? ircChannel : ircConversation;
-      this._conversations[name] = new constructor(this, aName, this._nickname);
+    if (!this.conversations.has(aName)) {
+      // If the whois information has been received, we have the proper nick
+      // capitalization.
+      if (this.whoisInformation.has(aName))
+        aName = this.whoisInformation.get(aName).nick;
+      let convClass = this.isMUCName(aName) ? ircChannel : ircConversation;
+      this.conversations.set(aName, new convClass(this, aName, this._nickname));
     }
-    return this._conversations[name];
+    return this.conversations.get(aName);
   },
 
   removeConversation: function(aConversationName) {
-    if (this.hasConversation(aConversationName))
-      delete this._conversations[this.normalize(aConversationName)];
+    if (this.conversations.has(aConversationName))
+      this.conversations.delete(aConversationName);
   },
 
   // This builds the message string that will be sent to the server.
-  buildMessage: function(aCommand, aParams) {
+  buildMessage: function(aCommand, aParams = []) {
     if (!aCommand) {
       this.ERROR("IRC messages must have a command.");
       return null;
@@ -1507,9 +1510,9 @@ ircAccount.prototype = {
     }
 
     let message = aCommand;
-    // If aParams is empty, then use an empty array. If aParams is not an array,
-    // consider it to be a single parameter and put it into an array.
-    let params = !aParams ? [] : Array.isArray(aParams) ? aParams : [aParams];
+    // If aParams is not an array, consider it to be a single parameter and put
+    // it into an array.
+    let params = Array.isArray(aParams) ? aParams : [aParams];
     if (params.length) {
       if (params.slice(0, -1).some(function(p) p.contains(" "))) {
         this.ERROR("IRC parameters cannot have spaces: " + params.slice(0, -1));
@@ -1579,12 +1582,12 @@ ircAccount.prototype = {
 
   // CTCP messages are \001<COMMAND> [<parameters>]*\001.
   // Returns false if the message could not be sent.
-  sendCTCPMessage: function(aCommand, aParams, aTarget, aIsNotice) {
+  sendCTCPMessage: function(aTarget, aIsNotice, aCtcpCommand, aParams = []) {
     // Combine the CTCP command and parameters into the single IRC param.
-    let ircParam = aCommand;
-    // If aParams is empty, then use an empty array. If aParams is not an array,
-    // consider it to be a single parameter and put it into an array.
-    let params = !aParams ? [] : Array.isArray(aParams) ? aParams : [aParams];
+    let ircParam = aCtcpCommand;
+    // If aParams is not an array, consider it to be a single parameter and put
+    // it into an array.
+    let params = Array.isArray(aParams) ? aParams : [aParams];
     if (params.length)
       ircParam += " " + params.join(" ");
 
@@ -1630,16 +1633,15 @@ ircAccount.prototype = {
     this.reportDisconnecting(aErrorReason, aErrorMessage);
 
     // Mark all contacts on the account as having an unknown status.
-    for each (let buddy in this._buddies)
-      buddy.setStatus(Ci.imIStatusInfo.STATUS_UNKNOWN, "");
+    this.buddies.forEach(function(aBuddy)
+      aBuddy.setStatus(Ci.imIStatusInfo.STATUS_UNKNOWN, ""));
   },
 
-  gotDisconnected: function(aError, aErrorMessage) {
+  gotDisconnected: function(aError = Ci.prplIAccount.NO_ERROR,
+                            aErrorMessage = "") {
     if (!this.imAccount || this.disconnected)
        return;
 
-    if (aError === undefined)
-      aError = Ci.prplIAccount.NO_ERROR;
     // If we are already disconnecting, this call to gotDisconnected
     // is when the server acknowledges our disconnection.
     // Otherwise it's because we lost the connection.
@@ -1655,14 +1657,17 @@ ircAccount.prototype = {
     delete this.isAuthenticated;
 
     // Clean up each conversation: mark as left and remove participant.
-    for each (let conversation in this._conversations) {
-      if (conversation.isChat && !conversation.left) {
-        // Remove the user's nick and mark the conversation as left as that's
-        // the final known state of the room.
-        conversation.removeParticipant(this._nickname, true);
-        conversation.left = true;
+    this.conversations.forEach(conversation => {
+      if (conversation.isChat) {
+        conversation.joining = false; // In case we never finished joining.
+        if (!conversation.left) {
+          // Remove the user's nick and mark the conversation as left as that's
+          // the final known state of the room.
+          conversation.removeParticipant(this._nickname);
+          conversation.left = true;
+        }
       }
-    }
+    });
 
     // If we disconnected during a pending LIST request, make sure callbacks
     // receive any remaining channels.
@@ -1670,18 +1675,16 @@ ircAccount.prototype = {
       this._sendRemainingRoomInfo();
 
     // Clear whois table.
-    this.whoisInformation = {};
+    this.whoisInformation.clear();
 
     this.reportDisconnected();
   },
 
   remove: function() {
-    for each (let conv in this._conversations)
-      conv.close();
-    delete this._conversations;
-    for each (let buddy in this._buddies)
-      buddy.remove();
-    delete this._buddies;
+    this.conversations.forEach(conv => conv.close());
+    delete this.conversations;
+    this.buddies.forEach(function(aBuddy) aBuddy.remove());
+    delete this.buddies;
   },
 
   unInit: function() {
