@@ -11,7 +11,6 @@
 #include "nsIMessenger.h"
 #include "nsIMsgDatabase.h"
 #include "nsIMsgHdr.h"
-#include "nsMsgLineBuffer.h" // for nsByteArray
 #include "MailNewsTypes.h"
 #include "nsTArray.h"
 #include "nsIDBChangeListener.h"
@@ -19,8 +18,8 @@
 #include "nsITreeBoxObject.h"
 #include "nsITreeSelection.h"
 #include "nsIMsgFolder.h"
+#include "nsIMsgThread.h"
 #include "nsIDateTimeFormat.h"
-#include "nsIMsgHeaderParser.h"
 #include "nsIDOMElement.h"
 #include "nsIAtom.h"
 #include "nsIImapIncomingServer.h"
@@ -38,6 +37,8 @@
 class nsVoidArray;
 
 typedef nsAutoTArray<nsMsgViewIndex, 1> nsMsgViewIndexArray;
+static_assert(nsMsgViewIndex(nsMsgViewIndexArray::NoIndex) ==
+  nsMsgViewIndex_None, "These need to be the same value.");
 
 enum eFieldType {
     kCollationKey,
@@ -118,19 +119,19 @@ public:
 protected:
   static nsrefcnt gInstanceCount;
 
-  static PRUnichar* kHighestPriorityString;
-  static PRUnichar* kHighPriorityString;
-  static PRUnichar* kLowestPriorityString;
-  static PRUnichar* kLowPriorityString;
-  static PRUnichar* kNormalPriorityString;
+  static char16_t* kHighestPriorityString;
+  static char16_t* kHighPriorityString;
+  static char16_t* kLowestPriorityString;
+  static char16_t* kLowPriorityString;
+  static char16_t* kNormalPriorityString;
 
   static nsIAtom* kJunkMsgAtom;
   static nsIAtom* kNotJunkMsgAtom;
 
-  static PRUnichar* kReadString;
-  static PRUnichar* kRepliedString;
-  static PRUnichar* kForwardedString;
-  static PRUnichar* kNewString;
+  static char16_t* kReadString;
+  static char16_t* kRepliedString;
+  static char16_t* kForwardedString;
+  static char16_t* kNewString;
 
   nsCOMPtr<nsITreeBoxObject> mTree;
   nsCOMPtr<nsITreeSelection> mTreeSelection;
@@ -276,8 +277,6 @@ protected:
   uint32_t GetSize(void) {return(m_keys.Length());}
 
   // notification api's
-  void  NoteChange(nsMsgViewIndex firstlineChanged, int32_t numChanged,
-                    nsMsgViewNotificationCodeValue changeType);
   void  NoteStartChange(nsMsgViewIndex firstlineChanged, int32_t numChanged,
                         nsMsgViewNotificationCodeValue changeType);
   void  NoteEndChange(nsMsgViewIndex firstlineChanged, int32_t numChanged,
@@ -348,7 +347,7 @@ protected:
   nsresult ToggleMessageKilled(nsMsgViewIndex * indices, int32_t numIndices, nsMsgViewIndex *resultIndex, bool *resultToggleState);
   bool OfflineMsgSelected(nsMsgViewIndex * indices, int32_t numIndices);
   bool NonDummyMsgSelected(nsMsgViewIndex * indices, int32_t numIndices);
-  PRUnichar * GetString(const PRUnichar *aStringName);
+  char16_t * GetString(const char16_t *aStringName);
   nsresult GetPrefLocalizedString(const char *aPrefName, nsString& aResult);
   nsresult GetLabelPrefStringAndAtom(const char *aPrefName, nsString& aColor, nsIAtom** aColorAtom);
   nsresult AppendKeywordProperties(const nsACString& keywords, nsAString& properties, bool addSelectedTextProperty);
@@ -391,6 +390,7 @@ protected:
   bool mIsXFVirtual;        // a virtual folder with multiple folders
 
   bool mShowSizeInLines;    // for news we show lines instead of size when true
+  bool mSortThreadsByRoot;  // as opposed to by the newest message
   bool m_sortValid;
   bool mSelectionSummarized;
   // we asked the front end to summarize the selection and it did not.
@@ -410,7 +410,6 @@ protected:
 
   // I18N date formatter service which we'll want to cache locally.
   nsCOMPtr<nsIDateTimeFormat> mDateFormatter;
-  nsCOMPtr<nsIMsgHeaderParser> mHeaderParser;
   nsCOMPtr<nsIMsgTagService> mTagService;
   nsWeakPtr mMessengerWeak;
   nsWeakPtr mMsgWindowWeak;
@@ -448,7 +447,7 @@ protected:
   nsCOMArray <nsIMsgCustomColumnHandler> m_customColumnHandlers;
   nsTArray<nsString> m_customColumnHandlerIDs;
   
-  nsIMsgCustomColumnHandler* GetColumnHandler(const PRUnichar*);
+  nsIMsgCustomColumnHandler* GetColumnHandler(const char16_t*);
   nsIMsgCustomColumnHandler* GetCurColumnHandlerFromDBInfo();
 
 #ifdef DEBUG_David_Bienvenu
