@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * David Bienvenu.
- * Portions created by the Initial Developer are Copyright (C) 2004
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   David Bienvenu <bienvenu@nventure.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "msgCore.h"
 #include "nsMsgGroupThread.h"
@@ -42,7 +9,7 @@
 #include "nsMsgMessageFlags.h"
 #include "nsMsgUtils.h"
 
-NS_IMPL_ISUPPORTS1(nsMsgGroupThread, nsIMsgThread)
+NS_IMPL_ISUPPORTS(nsMsgGroupThread, nsIMsgThread)
 
 nsMsgGroupThread::nsMsgGroupThread()
 {
@@ -83,14 +50,14 @@ NS_IMETHODIMP nsMsgGroupThread::GetThreadKey(nsMsgKey *aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgGroupThread::GetFlags(PRUint32 *aFlags)
+NS_IMETHODIMP nsMsgGroupThread::GetFlags(uint32_t *aFlags)
 {
   NS_ENSURE_ARG_POINTER(aFlags);
   *aFlags = m_flags;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgGroupThread::SetFlags(PRUint32 aFlags)
+NS_IMETHODIMP nsMsgGroupThread::SetFlags(uint32_t aFlags)
 {
   m_flags = aFlags;
   return NS_OK;
@@ -108,19 +75,19 @@ NS_IMETHODIMP nsMsgGroupThread::GetSubject(nsACString& result)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsMsgGroupThread::GetNumChildren(PRUint32 *aNumChildren)
+NS_IMETHODIMP nsMsgGroupThread::GetNumChildren(uint32_t *aNumChildren)
 {
   NS_ENSURE_ARG_POINTER(aNumChildren);
   *aNumChildren = m_keys.Length(); // - ((m_dummy) ? 1 : 0);
   return NS_OK;
 }
 
-PRUint32 nsMsgGroupThread::NumRealChildren()
+uint32_t nsMsgGroupThread::NumRealChildren()
 {
   return m_keys.Length() - ((m_dummy) ? 1 : 0);
 }
 
-NS_IMETHODIMP nsMsgGroupThread::GetNumUnreadChildren (PRUint32 *aNumUnreadChildren)
+NS_IMETHODIMP nsMsgGroupThread::GetNumUnreadChildren (uint32_t *aNumUnreadChildren)
 {
   NS_ENSURE_ARG_POINTER(aNumUnreadChildren);
   *aNumUnreadChildren = m_numUnreadChildren;
@@ -159,7 +126,7 @@ nsMsgViewIndex nsMsgGroupThread::AddMsgHdrInDateOrder(nsIMsgDBHdr *child, nsMsgD
 {
   nsMsgKey newHdrKey;
   child->GetMessageKey(&newHdrKey);
-  PRUint32 insertIndex = 0;
+  uint32_t insertIndex = 0;
   // since we're sorted by date, we could do a binary search for the 
   // insert point. Or, we could start at the end...
   if (m_keys.Length() > 0)
@@ -196,13 +163,13 @@ nsMsgGroupThread::GetInsertIndexFromView(nsMsgDBView *view,
                                           nsIMsgDBHdr *child, 
                                           nsMsgViewSortOrderValue threadSortOrder)
 {
-   return view->GetInsertIndexHelper(child, m_keys, nsnull, threadSortOrder, nsMsgViewSortType::byDate);
+   return view->GetInsertIndexHelper(child, m_keys, nullptr, threadSortOrder, nsMsgViewSortType::byDate);
 }
 
 nsMsgViewIndex nsMsgGroupThread::AddChildFromGroupView(nsIMsgDBHdr *child, nsMsgDBView *view)
 {
-  PRUint32 newHdrFlags = 0;
-  PRUint32 msgDate;
+  uint32_t newHdrFlags = 0;
+  uint32_t msgDate;
   nsMsgKey newHdrKey = 0;
   
   child->GetFlags(&newHdrFlags);
@@ -212,7 +179,7 @@ nsMsgViewIndex nsMsgGroupThread::AddChildFromGroupView(nsIMsgDBHdr *child, nsMsg
     SetNewestMsgDate(msgDate);
 
   child->AndFlags(~(nsMsgMessageFlags::Watched), &newHdrFlags);
-  PRUint32 numChildren;
+  uint32_t numChildren;
   
   // get the num children before we add the new header.
   GetNumChildren(&numChildren);
@@ -233,18 +200,18 @@ nsresult nsMsgGroupThread::ReparentNonReferenceChildrenOf(nsIMsgDBHdr *topLevelH
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgGroupThread::GetChildKeyAt(PRInt32 aIndex, nsMsgKey *aResult)
+NS_IMETHODIMP nsMsgGroupThread::GetChildKeyAt(uint32_t aIndex, nsMsgKey *aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
-  if (aIndex < 0 || (PRUint32)aIndex >= m_keys.Length())
+  if (aIndex >= m_keys.Length())
     return NS_ERROR_INVALID_ARG;
   *aResult = m_keys[aIndex];
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgGroupThread::GetChildHdrAt(PRInt32 aIndex, nsIMsgDBHdr **aResult)
+NS_IMETHODIMP nsMsgGroupThread::GetChildHdrAt(uint32_t aIndex, nsIMsgDBHdr **aResult)
 {
-  if (aIndex < 0 || (PRUint32)aIndex >= m_keys.Length())
+  if (aIndex >= m_keys.Length())
     return NS_MSG_MESSAGE_NOT_FOUND;
   return m_db->GetMsgHdrForKey(m_keys[aIndex], aResult);
 }
@@ -252,11 +219,13 @@ NS_IMETHODIMP nsMsgGroupThread::GetChildHdrAt(PRInt32 aIndex, nsIMsgDBHdr **aRes
 
 NS_IMETHODIMP nsMsgGroupThread::GetChild(nsMsgKey msgKey, nsIMsgDBHdr **aResult)
 {
-  return GetChildHdrAt((PRInt32) m_keys.IndexOf(msgKey), aResult);
+  return GetChildHdrAt(m_keys.IndexOf(msgKey), aResult);
 }
 
-NS_IMETHODIMP nsMsgGroupThread::RemoveChildAt(PRInt32 aIndex)
+NS_IMETHODIMP nsMsgGroupThread::RemoveChildAt(uint32_t aIndex)
 {
+  NS_ENSURE_TRUE(aIndex < m_keys.Length(), NS_MSG_MESSAGE_NOT_FOUND);
+
   m_keys.RemoveElementAt(aIndex);
   return NS_OK;
 }
@@ -269,17 +238,16 @@ nsresult nsMsgGroupThread::RemoveChild(nsMsgKey msgKey)
 
 NS_IMETHODIMP nsMsgGroupThread::RemoveChildHdr(nsIMsgDBHdr *child, nsIDBChangeAnnouncer *announcer)
 {
-  PRUint32 flags;
+  NS_ENSURE_ARG_POINTER(child);
+
+  uint32_t flags;
   nsMsgKey key;
-  
-  if (!child)
-    return NS_ERROR_NULL_POINTER;
-  
+
   child->GetFlags(&flags);
   child->GetMessageKey(&key);
-  
+
   // if this was the newest msg, clear the newest msg date so we'll recalc.
-  PRUint32 date;
+  uint32_t date;
   child->GetDateInSeconds(&date);
   if (date == m_newestMsgDate)
     SetNewestMsgDate(0);
@@ -294,7 +262,8 @@ NS_IMETHODIMP nsMsgGroupThread::RemoveChildHdr(nsIMsgDBHdr *child, nsIDBChangeAn
   if (m_dummy && wasFirstChild && m_keys.Length() > 1)
   {
     nsIMsgDBHdr *newRootChild;
-    GetChildHdrAt(1, &newRootChild);
+    rv = GetChildHdrAt(1, &newRootChild);
+    NS_ENSURE_SUCCESS(rv, rv);
     SetMsgHdrAt(0, newRootChild);
   }
 
@@ -305,8 +274,8 @@ nsresult nsMsgGroupThread::ReparentChildrenOf(nsMsgKey oldParent, nsMsgKey newPa
 {
   nsresult rv = NS_OK;
   
-  PRUint32 numChildren;
-  PRUint32 childIndex = 0;
+  uint32_t numChildren;
+  uint32_t childIndex = 0;
   
   GetNumChildren(&numChildren);
   
@@ -328,7 +297,7 @@ nsresult nsMsgGroupThread::ReparentChildrenOf(nsMsgKey oldParent, nsMsgKey newPa
           curHdr->SetThreadParent(newParent);
           curHdr->GetMessageKey(&curKey);
           if (announcer)
-            announcer->NotifyParentChangedAll(curKey, oldParent, newParent, nsnull);
+            announcer->NotifyParentChangedAll(curKey, oldParent, newParent, nullptr);
           // if the old parent was the root of the thread, then only the first child gets 
           // promoted to root, and other children become children of the new root.
           if (newParent == nsMsgKey_None)
@@ -362,7 +331,7 @@ public:
   
   nsMsgGroupThreadEnumerator(nsMsgGroupThread *thread, nsMsgKey startKey,
   nsMsgGroupThreadEnumeratorFilter filter, void* closure);
-  PRInt32 MsgKeyFirstChildIndex(nsMsgKey inMsgKey);
+  int32_t MsgKeyFirstChildIndex(nsMsgKey inMsgKey);
   virtual ~nsMsgGroupThreadEnumerator();
   
 protected:
@@ -373,7 +342,7 @@ protected:
   nsMsgGroupThread*       mThread;
   nsMsgKey                mThreadParentKey;
   nsMsgKey                mFirstMsgKey;
-  PRInt32                 mChildIndex;
+  int32_t                 mChildIndex;
   bool                    mDone;
   bool                    mNeedToPrefetch;
   nsMsgGroupThreadEnumeratorFilter     mFilter;
@@ -392,18 +361,18 @@ nsMsgGroupThreadEnumerator::nsMsgGroupThreadEnumerator(nsMsgGroupThread *thread,
   mNeedToPrefetch = true;
   mFirstMsgKey = nsMsgKey_None;
   
-  nsresult rv = mThread->GetRootHdr(nsnull, getter_AddRefs(mResultHdr));
+  nsresult rv = mThread->GetRootHdr(nullptr, getter_AddRefs(mResultHdr));
   
   if (NS_SUCCEEDED(rv) && mResultHdr)
     mResultHdr->GetMessageKey(&mFirstMsgKey);
   
-  PRUint32 numChildren;
+  uint32_t numChildren;
   mThread->GetNumChildren(&numChildren);
   
   if (mThreadParentKey != nsMsgKey_None)
   {
     nsMsgKey msgKey = nsMsgKey_None;
-    PRUint32 childIndex = 0;
+    uint32_t childIndex = 0;
     
     
     for (childIndex = 0; childIndex < numChildren; childIndex++)
@@ -431,7 +400,7 @@ nsMsgGroupThreadEnumerator::nsMsgGroupThreadEnumerator(nsMsgGroupThread *thread,
   
 #ifdef DEBUG_bienvenu1
   nsCOMPtr <nsIMsgDBHdr> child;
-  for (PRUint32 childIndex = 0; childIndex < numChildren; childIndex++)
+  for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
     rv = mThread->GetChildHdrAt(childIndex, getter_AddRefs(child));
     if (NS_SUCCEEDED(rv) && child)
@@ -456,10 +425,10 @@ nsMsgGroupThreadEnumerator::~nsMsgGroupThreadEnumerator()
     NS_RELEASE(mThread);
 }
 
-NS_IMPL_ISUPPORTS1(nsMsgGroupThreadEnumerator, nsISimpleEnumerator)
+NS_IMPL_ISUPPORTS(nsMsgGroupThreadEnumerator, nsISimpleEnumerator)
 
 
-PRInt32 nsMsgGroupThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
+int32_t nsMsgGroupThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
 {
   //	if (msgKey != mThreadParentKey)
   //		mDone = true;
@@ -467,9 +436,9 @@ PRInt32 nsMsgGroupThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
   // If the inMsgKey is the first message in the thread, then all children
   // without parents are considered to be children of inMsgKey.
   // Otherwise, only true children qualify.
-  PRUint32 numChildren;
+  uint32_t numChildren;
   nsCOMPtr <nsIMsgDBHdr> curHdr;
-  PRInt32 firstChildIndex = -1;
+  int32_t firstChildIndex = -1;
   
   mThread->GetNumChildren(&numChildren);
   
@@ -478,7 +447,7 @@ PRInt32 nsMsgGroupThreadEnumerator::MsgKeyFirstChildIndex(nsMsgKey inMsgKey)
   //	if (inMsgKey == mThread->m_threadRootKey)
   //		return (numChildren > 1) ? 1 : -1;
   
-  for (PRUint32 curChildIndex = 0; curChildIndex < numChildren; curChildIndex++)
+  for (uint32_t curChildIndex = 0; curChildIndex < numChildren; curChildIndex++)
   {
     nsresult rv = mThread->GetChildHdrAt(curChildIndex, getter_AddRefs(curHdr));
     if (NS_SUCCEEDED(rv) && curHdr)
@@ -520,7 +489,7 @@ NS_IMETHODIMP nsMsgGroupThreadEnumerator::GetNext(nsISupports **aItem)
 nsresult nsMsgGroupThreadEnumerator::Prefetch()
 {
   nsresult rv=NS_OK;          // XXX or should this default to an error?
-  mResultHdr = nsnull;
+  mResultHdr = nullptr;
   if (mThreadParentKey == nsMsgKey_None)
   {
     rv = mThread->GetRootHdr(&mChildIndex, getter_AddRefs(mResultHdr));
@@ -529,10 +498,10 @@ nsresult nsMsgGroupThreadEnumerator::Prefetch()
   }
   else if (!mDone)
   {
-    PRUint32 numChildren;
+    uint32_t numChildren;
     mThread->GetNumChildren(&numChildren);
     
-    while (mChildIndex < (PRInt32) numChildren)
+    while ((uint32_t)mChildIndex < numChildren)
     {
       rv  = mThread->GetChildHdrAt(mChildIndex++, getter_AddRefs(mResultHdr));
       if (NS_SUCCEEDED(rv) && mResultHdr)
@@ -541,7 +510,7 @@ nsresult nsMsgGroupThreadEnumerator::Prefetch()
         nsMsgKey curKey;
         
         if (mFilter && NS_FAILED(mFilter(mResultHdr, mClosure))) {
-          mResultHdr = nsnull;
+          mResultHdr = nullptr;
           continue;
         }
         
@@ -554,7 +523,7 @@ nsresult nsMsgGroupThreadEnumerator::Prefetch()
           (parentKey == nsMsgKey_None 
           && mThreadParentKey == mFirstMsgKey && curKey != mThreadParentKey))
           break;
-        mResultHdr = nsnull;
+        mResultHdr = nullptr;
       }
       else
         NS_ASSERTION(false, "better be able to get child");
@@ -599,8 +568,8 @@ NS_IMETHODIMP nsMsgGroupThreadEnumerator::HasMoreElements(bool *aResult)
 
 NS_IMETHODIMP nsMsgGroupThread::EnumerateMessages(nsMsgKey parentKey, nsISimpleEnumerator* *result)
 {
-    nsMsgGroupThreadEnumerator* e = new nsMsgGroupThreadEnumerator(this, parentKey, nsnull, nsnull);
-    if (e == nsnull)
+    nsMsgGroupThreadEnumerator* e = new nsMsgGroupThreadEnumerator(this, parentKey, nullptr, nullptr);
+    if (e == nullptr)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(e);
     *result = e;
@@ -608,12 +577,12 @@ NS_IMETHODIMP nsMsgGroupThread::EnumerateMessages(nsMsgKey parentKey, nsISimpleE
     return NS_OK;
 }
 #if 0
-nsresult nsMsgGroupThread::ReparentMsgsWithInvalidParent(PRUint32 numChildren, nsMsgKey threadParentKey)
+nsresult nsMsgGroupThread::ReparentMsgsWithInvalidParent(uint32_t numChildren, nsMsgKey threadParentKey)
 {
   nsresult ret = NS_OK;
   // run through looking for messages that don't have a correct parent, 
   // i.e., a parent that's in the thread!
-  for (PRInt32 childIndex = 0; childIndex < (PRInt32) numChildren; childIndex++)
+  for (int32_t childIndex = 0; childIndex < (int32_t) numChildren; childIndex++)
   {
     nsCOMPtr <nsIMsgDBHdr> curChild;
     ret  = GetChildHdrAt(childIndex, getter_AddRefs(curChild));
@@ -635,12 +604,12 @@ nsresult nsMsgGroupThread::ReparentMsgsWithInvalidParent(PRUint32 numChildren, n
   return ret;
 }
 #endif
-NS_IMETHODIMP nsMsgGroupThread::GetRootHdr(PRInt32 *resultIndex, nsIMsgDBHdr **result)
+NS_IMETHODIMP nsMsgGroupThread::GetRootHdr(int32_t *resultIndex, nsIMsgDBHdr **result)
 {
   if (!result)
     return NS_ERROR_NULL_POINTER;
   
-  *result = nsnull;
+  *result = nullptr;
   
   if (m_threadRootKey != nsMsgKey_None)
   {
@@ -650,11 +619,11 @@ NS_IMETHODIMP nsMsgGroupThread::GetRootHdr(PRInt32 *resultIndex, nsIMsgDBHdr **r
     else
     {
       printf("need to reset thread root key\n");
-      PRUint32 numChildren;
+      uint32_t numChildren;
       nsMsgKey threadParentKey = nsMsgKey_None;
       GetNumChildren(&numChildren);
       
-      for (PRInt32 childIndex = 0; childIndex < (PRInt32) numChildren; childIndex++)
+      for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
       {
         nsCOMPtr <nsIMsgDBHdr> curChild;
         ret  = GetChildHdrAt(childIndex, getter_AddRefs(curChild));
@@ -691,26 +660,23 @@ NS_IMETHODIMP nsMsgGroupThread::GetRootHdr(PRInt32 *resultIndex, nsIMsgDBHdr **r
   return GetChildHdrAt(0, result);
 }
 
-nsresult nsMsgGroupThread::ChangeUnreadChildCount(PRInt32 delta)
+nsresult nsMsgGroupThread::ChangeUnreadChildCount(int32_t delta)
 {
   m_numUnreadChildren += delta;
   return NS_OK;
 }
 
-nsresult nsMsgGroupThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **result, PRInt32 *resultIndex)
+nsresult nsMsgGroupThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **result, int32_t *resultIndex)
 {
-  PRUint32 numChildren;
-  PRUint32 childIndex = 0;
+  uint32_t numChildren;
+  uint32_t childIndex = 0;
   nsresult rv = NS_OK;        // XXX or should this default to an error?
   
   if (!result)
     return NS_ERROR_NULL_POINTER;
   
   GetNumChildren(&numChildren);
-  
-  if ((PRInt32) numChildren < 0)
-    numChildren = 0;
-  
+
   for (childIndex = 0; childIndex < numChildren; childIndex++)
   {
     rv = GetChildHdrAt(childIndex, result);
@@ -728,7 +694,7 @@ nsresult nsMsgGroupThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **
     }
   }
   if (resultIndex)
-    *resultIndex = childIndex;
+    *resultIndex = (int32_t) childIndex;
   
   return rv;
 }
@@ -736,15 +702,15 @@ nsresult nsMsgGroupThread::GetChildHdrForKey(nsMsgKey desiredKey, nsIMsgDBHdr **
 NS_IMETHODIMP nsMsgGroupThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
 {
   NS_ENSURE_ARG(result);
-  PRUint32 numChildren;
+  uint32_t numChildren;
   nsresult rv = NS_OK;
   
   GetNumChildren(&numChildren);
   
-  if ((PRInt32) numChildren < 0)
+  if ((int32_t) numChildren < 0)
     numChildren = 0;
   
-  for (PRUint32 childIndex = 0; childIndex < numChildren; childIndex++)
+  for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
   {
     nsCOMPtr <nsIMsgDBHdr> child;
     rv = GetChildHdrAt(childIndex, getter_AddRefs(child));
@@ -767,26 +733,23 @@ NS_IMETHODIMP nsMsgGroupThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
   return rv;
 }
 
-NS_IMETHODIMP nsMsgGroupThread::GetNewestMsgDate(PRUint32 *aResult) 
+NS_IMETHODIMP nsMsgGroupThread::GetNewestMsgDate(uint32_t *aResult) 
 {
   // if this hasn't been set, figure it out by enumerating the msgs in the thread.
   if (!m_newestMsgDate)
   {
-    PRUint32 numChildren;
+    uint32_t numChildren;
     nsresult rv = NS_OK;
   
     GetNumChildren(&numChildren);
   
-    if ((PRInt32) numChildren < 0)
-      numChildren = 0;
-  
-    for (PRUint32 childIndex = 0; childIndex < numChildren; childIndex++)
+    for (uint32_t childIndex = 0; childIndex < numChildren; childIndex++)
     {
       nsCOMPtr <nsIMsgDBHdr> child;
       rv = GetChildHdrAt(childIndex, getter_AddRefs(child));
       if (NS_SUCCEEDED(rv) && child)
       {
-        PRUint32 msgDate;
+        uint32_t msgDate;
         child->GetDateInSeconds(&msgDate);
         if (msgDate > m_newestMsgDate)
           m_newestMsgDate = msgDate;
@@ -799,7 +762,7 @@ NS_IMETHODIMP nsMsgGroupThread::GetNewestMsgDate(PRUint32 *aResult)
 }
 
 
-NS_IMETHODIMP nsMsgGroupThread::SetNewestMsgDate(PRUint32 aNewestMsgDate) 
+NS_IMETHODIMP nsMsgGroupThread::SetNewestMsgDate(uint32_t aNewestMsgDate) 
 {
   m_newestMsgDate = aNewestMsgDate;
   return NS_OK;
@@ -813,31 +776,33 @@ nsMsgXFGroupThread::~nsMsgXFGroupThread()
 {
 }
 
-NS_IMETHODIMP nsMsgXFGroupThread::GetNumChildren(PRUint32 *aNumChildren)
+NS_IMETHODIMP nsMsgXFGroupThread::GetNumChildren(uint32_t *aNumChildren)
 {
   NS_ENSURE_ARG_POINTER(aNumChildren);
-  *aNumChildren = m_folders.Count();
+  *aNumChildren = m_folders.Length();
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgXFGroupThread::GetChildHdrAt(PRInt32 aIndex, nsIMsgDBHdr **aResult)
+NS_IMETHODIMP nsMsgXFGroupThread::GetChildHdrAt(uint32_t aIndex, nsIMsgDBHdr **aResult)
 {
-  if (aIndex < 0 || aIndex >= m_folders.Count())
+  if (aIndex >= m_folders.Length())
     return NS_MSG_MESSAGE_NOT_FOUND;
-  m_folders.ObjectAt(aIndex)->GetMessageHeader(m_keys[aIndex], aResult);
-  return NS_OK;
+  return m_folders.ObjectAt(aIndex)->GetMessageHeader(m_keys[aIndex], aResult);
 }
 
-NS_IMETHODIMP nsMsgXFGroupThread::GetChildKeyAt(PRInt32 aIndex, nsMsgKey *aResult)
+NS_IMETHODIMP nsMsgXFGroupThread::GetChildKeyAt(uint32_t aIndex, nsMsgKey *aResult)
 {
   NS_ASSERTION(false, "shouldn't call this");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsMsgXFGroupThread::RemoveChildAt(PRInt32 aIndex)
+NS_IMETHODIMP nsMsgXFGroupThread::RemoveChildAt(uint32_t aIndex)
 {
-  nsMsgGroupThread::RemoveChildAt(aIndex);
-  m_folders.RemoveObjectAt(aIndex);
+  NS_ENSURE_TRUE(aIndex < m_folders.Length(), NS_MSG_MESSAGE_NOT_FOUND);
+
+  nsresult rv = nsMsgGroupThread::RemoveChildAt(aIndex);
+  NS_ENSURE_SUCCESS(rv, rv);
+  m_folders.RemoveElementAt(aIndex);
   return NS_OK;
 }
 
@@ -863,7 +828,7 @@ nsMsgViewIndex nsMsgXFGroupThread::FindMsgHdr(nsIMsgDBHdr *hdr)
   hdr->GetMessageKey(&msgKey);
   nsCOMPtr<nsIMsgFolder> folder;
   hdr->GetFolder(getter_AddRefs(folder));
-  PRUint32 index = 0;
+  size_t index = 0;
   while (true) {
     index = m_keys.IndexOf(msgKey, index);
     if (index == m_keys.NoIndex || m_folders[index] == folder)
