@@ -11,6 +11,7 @@
 #include "nsStringGlue.h"
 #include "nsIServiceManager.h"
 #include "nsCOMPtr.h"
+#include "nsIFolderLookupService.h"
 #include "nsIImapUrl.h"
 #include "nsIMailboxUrl.h"
 #include "nsINntpUrl.h"
@@ -367,8 +368,6 @@ static bool ConvertibleToNative(const nsAutoString& str)
   const static uint32_t MAX_LEN = 55;
 #elif defined(XP_WIN32)
   const static uint32_t MAX_LEN = 55;
-#elif defined(XP_OS2)
-  const static uint32_t MAX_LEN = 55;
 #else
   #error need_to_define_your_max_filename_length
 #endif
@@ -566,9 +565,10 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *aFolderURI,
     ? oldPath.FindChar('/', startSlashPos + 1) - 1 : oldPath.Length() - 1;
   if (endSlashPos < 0)
     endSlashPos = oldPath.Length();
-#ifdef XP_MACOSX
-  bool isMailboxUri = aScheme.EqualsLiteral("none") ||
-                        aScheme.EqualsLiteral("pop3");
+#if defined(XP_UNIX) || defined(XP_MACOSX)
+  bool isLocalUri = aScheme.EqualsLiteral("none") ||
+                    aScheme.EqualsLiteral("pop3") ||
+                    aScheme.EqualsLiteral("rss");
 #endif
   // trick to make sure we only add the path to the first n-1 folders
   bool haveFirst=false;
@@ -590,11 +590,11 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *aFolderURI,
           CopyUTF16toMUTF7(pathPiece, tmp);
           CopyASCIItoUTF16(tmp, pathPiece);
       }
-#ifdef XP_MACOSX
+#if defined(XP_UNIX) || defined(XP_MACOSX)
       // Don't hash path pieces because local mail folder uri's have already
       // been hashed. We're only doing this on the mac to limit potential
       // regressions.
-      if (!isMailboxUri)
+      if (!isLocalUri)
 #endif
       NS_MsgHashIfNecessary(pathPiece);
       path += pathPiece;
@@ -721,11 +721,15 @@ bool NS_MsgStripRE(const char **stringP, uint32_t *lengthP, char **modifiedSubje
           char charset[nsIMimeConverter::MAX_CHARSET_NAME_LENGTH] = "";
           if (nsIMimeConverter::MAX_CHARSET_NAME_LENGTH >= (p2 - p1))
             strncpy(charset, p1, p2 - p1);
-          rv = mimeConverter->EncodeMimePartIIStr_UTF8(nsDependentCString(s), false, charset,
-            sizeof("Subject:"), nsIMimeConverter::MIME_ENCODED_WORD_SIZE,
-            modifiedSubject);
+          nsAutoCString encodedString;
+          rv = mimeConverter->EncodeMimePartIIStr_UTF8(nsDependentCString(s),
+            false, charset, sizeof("Subject:"),
+            nsIMimeConverter::MIME_ENCODED_WORD_SIZE, encodedString);
           if (NS_SUCCEEDED(rv))
+          {
+            *modifiedSubject = PL_strdup(encodedString.get());
             return result;
+          }
         }
       }
     }
@@ -816,27 +820,13 @@ nsresult GetExistingFolder(const nsCString& aFolderURI, nsIMsgFolder **aFolder)
   *aFolder = nullptr;
 
   nsresult rv;
-  nsCOMPtr<nsIRDFService> rdf(do_GetService("@mozilla.org/rdf/rdf-service;1", &rv));
+  nsCOMPtr<nsIFolderLookupService> fls(do_GetService(NSIFLS_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIRDFResource> resource;
-  rv = rdf->GetResource(aFolderURI, getter_AddRefs(resource));
+  rv = fls->GetFolderForURL(aFolderURI, aFolder);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr <nsIMsgFolder> thisFolder;
-  thisFolder = do_QueryInterface(resource, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Parent doesn't exist means that this folder doesn't exist.
-  nsCOMPtr<nsIMsgFolder> parentFolder;
-  rv = thisFolder->GetParent(getter_AddRefs(parentFolder));
-  if (NS_SUCCEEDED(rv)) {
-    // When parentFolder is null with NS_OK, we should return error.
-    NS_ENSURE_TRUE(parentFolder, NS_ERROR_FAILURE);
-
-    NS_ADDREF(*aFolder = thisFolder);
-  }
-  return rv;
+  return *aFolder ? NS_OK : NS_ERROR_FAILURE;
 }
 
 bool IsAFromSpaceLine(char *start, const char *end)
@@ -1315,6 +1305,28 @@ NS_MSG_BASE nsresult NS_GetLocalizedUnicharPreferenceWithDefault(nsIPrefBranch *
     return NS_OK;
 }
 
+NS_MSG_BASE nsresult NS_GetLocalizedUnicharPreference(nsIPrefBranch *prefBranch,  //can be null, if so uses the root branch
+                                                      const char *prefName,
+                                                      nsAString& prefValue)
+{
+  NS_ENSURE_ARG_POINTER(prefName);
+
+  nsCOMPtr<nsIPrefBranch> pbr;
+  if (!prefBranch) {
+    pbr = do_GetService(NS_PREFSERVICE_CONTRACTID);
+    prefBranch = pbr;
+  }
+
+  nsCOMPtr<nsIPrefLocalizedString> str;
+  nsresult rv = prefBranch->GetComplexValue(prefName, NS_GET_IID(nsIPrefLocalizedString), getter_AddRefs(str));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsString tmpValue;
+  str->ToString(getter_Copies(tmpValue));
+  prefValue.Assign(tmpValue);
+  return NS_OK;
+}
+
 void PRTime2Seconds(PRTime prTime, uint32_t *seconds)
 {
   *seconds = (uint32_t)(prTime / PR_USEC_PER_SEC);
@@ -1545,7 +1557,9 @@ nsresult MsgGetLocalFileFromURI(const nsACString &aUTF8Path, nsIFile **aFile)
   nsCOMPtr<nsIFile> argFile;
   rv = argFileURL->GetFile(getter_AddRefs(argFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  return CallQueryInterface(argFile, aFile);
+
+  argFile.forget(aFile);
+  return NS_OK;
 }
 
 #ifndef MOZILLA_INTERNAL_API
@@ -1568,21 +1582,21 @@ NS_MSG_BASE bool MsgIsUTF8(const nsACString& aString)
 
   while (ptr < done_reading) {
     uint8_t c;
-    
+
     if (0 == state) {
 
       c = *ptr++;
 
-      if ((c & 0x80) == 0x00) 
+      if ((c & 0x80) == 0x00)
         continue;
 
       if ( c <= 0xC1 ) // [80-BF] where not expected, [C0-C1] for overlong.
         return false;
-      else if ((c & 0xE0) == 0xC0) 
+      else if ((c & 0xE0) == 0xC0)
         state = 1;
       else if ((c & 0xF0) == 0xE0) {
         state = 2;
-        if ( c == 0xE0 ) { // to exclude E0[80-9F][80-BF] 
+        if ( c == 0xE0 ) { // to exclude E0[80-9F][80-BF]
           overlong = true;
           olupper = 0x9F;
         } else if ( c == 0xED ) { // ED[A0-BF][80-BF] : surrogate codepoint
@@ -1597,7 +1611,7 @@ NS_MSG_BASE bool MsgIsUTF8(const nsACString& aString)
           overlong = true;
           olupper = 0x8F;
         }
-        else if ( c == 0xF4 ) { // to exclude F4[90-BF][80-BF] 
+        else if ( c == 0xF4 ) { // to exclude F4[90-BF][80-BF]
           // actually not surrogates but codepoints beyond 0x10FFFF
           surrogate = true;
           slower = 0x90;
@@ -1605,7 +1619,7 @@ NS_MSG_BASE bool MsgIsUTF8(const nsACString& aString)
       } else
         return false; // Not UTF-8 string
     }
-    
+
     while (ptr < done_reading && state) {
       c = *ptr++;
       --state;
@@ -1622,7 +1636,7 @@ NS_MSG_BASE bool MsgIsUTF8(const nsACString& aString)
       overlong = surrogate = false;
     }
   }
-  return !state; // state != 0 at the end indicates an invalid UTF-8 seq. 
+  return !state; // state != 0 at the end indicates an invalid UTF-8 seq.
 }
 
 #endif
@@ -1638,21 +1652,18 @@ NS_MSG_BASE void MsgStripQuotedPrintable (unsigned char *src)
 
   while (src[srcIdx] != 0)
   {
+    // Decode sequence of '=XY' into a character with code XY.
     if (src[srcIdx] == '=')
     {
-      unsigned char *token = &src[srcIdx];
-      unsigned char c = 0;
-
-      // decode the first quoted char
-      if (token[1] >= '0' && token[1] <= '9')
-        c = token[1] - '0';
-      else if (token[1] >= 'A' && token[1] <= 'F')
-        c = token[1] - ('A' - 10);
-      else if (token[1] >= 'a' && token[1] <= 'f')
-        c = token[1] - ('a' - 10);
+      if (MsgIsHex((const char*)src + srcIdx + 1, 2)) {
+        // If we got here, we successfully decoded a quoted printable sequence,
+        // so bump each pointer past it and move on to the next char.
+        dest[destIdx++] = MsgUnhex((const char*)src + srcIdx + 1, 2);
+        srcIdx += 3;
+      }
       else
       {
-        // first char after '=' isn't hex. check if it's a normal char
+        // If first char after '=' isn't hex check if it's a normal char
         // or a soft line break. If it's a soft line break, eat the
         // CR/LF/CRLF.
         if (src[srcIdx + 1] == '\r' || src[srcIdx + 1] == '\n')
@@ -1665,33 +1676,12 @@ NS_MSG_BASE void MsgStripQuotedPrintable (unsigned char *src)
               srcIdx++;
           }
         }
-        else // normal char, copy it.
+        else // The first or second char after '=' isn't hex, just copy the '='.
         {
-          dest[destIdx++] = src[srcIdx++]; // aka token[0]
+          dest[destIdx++] = src[srcIdx++];
         }
         continue;
       }
-
-      // decode the second quoted char
-      c = (c << 4);
-      if (token[2] >= '0' && token[2] <= '9')
-        c += token[2] - '0';
-      else if (token[2] >= 'A' && token[2] <= 'F')
-        c += token[2] - ('A' - 10);
-      else if (token[2] >= 'a' && token[2] <= 'f')
-        c += token[2] - ('a' - 10);
-      else
-      {
-        // second char after '=' isn't hex. copy the '=' as a normal char and keep going
-        dest[destIdx++] = src[srcIdx++]; // aka token[0]
-        continue;
-      }
-
-      // if we got here, we successfully decoded a quoted printable sequence,
-      // so bump each pointer past it and move on to the next char;
-      dest[destIdx++] = c;
-      srcIdx += 3;
-
     }
     else
       dest[destIdx++] = src[srcIdx++];
@@ -1710,7 +1700,7 @@ NS_MSG_BASE nsresult MsgEscapeString(const nsACString &aStr,
   return nu->EscapeString(aStr, aType, aResult);
 }
 
-NS_MSG_BASE nsresult MsgUnescapeString(const nsACString &aStr, uint32_t aFlags, 
+NS_MSG_BASE nsresult MsgUnescapeString(const nsACString &aStr, uint32_t aFlags,
                                        nsACString &aResult)
 {
   nsresult rv;
@@ -1811,7 +1801,7 @@ NS_MSG_BASE char16_t *MsgEscapeHTML2(const char16_t *aSourceBuffer,
 
   char16_t *resultBuffer = (char16_t *)nsMemory::Alloc(aSourceBufferLen *
                             6 * sizeof(char16_t) + sizeof(char16_t('\0')));
-                                                        
+
   char16_t *ptr = resultBuffer;
 
   if (resultBuffer) {
@@ -1876,7 +1866,7 @@ NS_MSG_BASE void MsgCompressWhitespace(nsCString& aString)
 
     // Loop through the white space
     char *wend = cur + 2;
-    while (IS_SPACE(*wend)) 
+    while (IS_SPACE(*wend))
       ++wend;
 
     uint32_t wlen = wend - cur - 1;
@@ -1894,6 +1884,7 @@ NS_MSG_BASE void MsgCompressWhitespace(nsCString& aString)
   aString.SetLength(end - start);
 }
 
+
 NS_MSG_BASE void MsgReplaceChar(nsString& str, const char *set, const char16_t replacement)
 {
   char16_t *c_str = str.BeginWriting();
@@ -1982,7 +1973,7 @@ public:
 };
 
 // XXX This needs to support threadsafe refcounting until we fix bug 243591.
-NS_IMPL_ISUPPORTS1(MsgInterfaceRequestorAgg, nsIInterfaceRequestor)
+NS_IMPL_ISUPPORTS(MsgInterfaceRequestorAgg, nsIInterfaceRequestor)
 
 NS_IMETHODIMP
 MsgInterfaceRequestorAgg::GetInterface(const nsIID &aIID, void **aResult)
@@ -2084,7 +2075,7 @@ MsgExamineForProxy(const char *scheme, const char *host,
     spec.AppendInt(port);
     // XXXXX - Under no circumstances whatsoever should any code which
     // wants a uri do this. I do this here because I do not, in fact,
-    // actually want a uri (the dummy uris created here may not be 
+    // actually want a uri (the dummy uris created here may not be
     // syntactically valid for the specific protocol), and all we need
     // is something which has a valid scheme, hostname, and a string
     // to pass to PAC if needed - bbaetz
@@ -2207,7 +2198,7 @@ NS_MSG_BASE nsresult MsgTermListToString(nsISupportsArray *aTermList, nsCString
 
     rv = term->GetTermAsString(stream);
     NS_ENSURE_SUCCESS(rv, rv);
-    
+
     aOutString += stream;
     aOutString += ')';
   }
@@ -2226,6 +2217,42 @@ NS_MSG_BASE uint64_t ParseUint64Str(const char *str)
 #endif
 }
 
+NS_MSG_BASE uint64_t MsgUnhex(const char *aHexString, size_t aNumChars)
+{
+  // Large numbers will not fit into uint64_t.
+  NS_ASSERTION(aNumChars <= 16, "Hex literal too long to convert!");
+
+  uint64_t result = 0;
+  for (size_t i = 0; i < aNumChars; i++)
+  {
+    unsigned char c = aHexString[i];
+    uint8_t digit;
+    if ((c >= '0') && (c <= '9'))
+      digit = (c - '0');
+    else if ((c >= 'a') && (c <= 'f'))
+      digit = ((c - 'a') + 10);
+    else if ((c >= 'A') && (c <= 'F'))
+      digit = ((c - 'A') + 10);
+    else
+      break;
+
+    result = (result << 4) | digit;
+  }
+
+  return result;
+}
+
+NS_MSG_BASE bool MsgIsHex(const char *aHexString, size_t aNumChars)
+{
+  for (size_t i = 0; i < aNumChars; i++)
+  {
+    if (!isxdigit(aHexString[i]))
+      return false;
+  }
+  return true;
+}
+
+
 NS_MSG_BASE nsresult
 MsgStreamMsgHeaders(nsIInputStream *aInputStream, nsIStreamListener *aConsumer)
 {
@@ -2278,7 +2305,7 @@ private:
   nsCString mCharset;
 };
 
-NS_IMPL_ISUPPORTS1(CharsetDetectionObserver, nsICharsetDetectionObserver)
+NS_IMPL_ISUPPORTS(CharsetDetectionObserver, nsICharsetDetectionObserver)
 
 NS_MSG_BASE nsresult
 MsgDetectCharsetFromFile(nsIFile *aFile, nsACString &aCharset)
@@ -2308,7 +2335,7 @@ MsgDetectCharsetFromFile(nsIFile *aFile, nsACString &aCharset)
   if (detector) {
     nsAutoCString buffer;
 
-    nsCOMPtr<CharsetDetectionObserver> observer = new CharsetDetectionObserver();
+    nsRefPtr<CharsetDetectionObserver> observer = new CharsetDetectionObserver();
 
     rv = detector->Init(observer);
     NS_ENSURE_SUCCESS(rv, rv);
