@@ -123,7 +123,7 @@ nsMsgComposeService::nsMsgComposeService()
   mCachedWindows = nullptr;
 }
 
-NS_IMPL_ISUPPORTS4(nsMsgComposeService,
+NS_IMPL_ISUPPORTS(nsMsgComposeService,
                    nsIMsgComposeService,
                    nsIObserver,
                    ICOMMANDLINEHANDLER,
@@ -896,7 +896,7 @@ public:
   char mLastBlockChars[3];
 };
 
-NS_IMPL_ISUPPORTS3(nsMsgTemplateReplyHelper,
+NS_IMPL_ISUPPORTS(nsMsgTemplateReplyHelper,
                    nsIStreamListener,
                    nsIRequestObserver,
                    nsIUrlListener)
@@ -1385,7 +1385,7 @@ nsresult nsMsgComposeService::AddGlobalHtmlDomains()
       // Get the current plaintext domain list into new list var
       ParseString(currentPlaintextDomainList, DOMAIN_DELIMITER, domainArray);
 
-      uint32_t i = domainArray.Length();
+      size_t i = domainArray.Length();
       if (i > 0) {
         // Append each domain in the preconfigured html domain list
         globalHtmlDomainList.StripWhitespace();
