@@ -185,7 +185,7 @@ TokenHash::TokenHash(uint32_t aEntrySize)
     mEntrySize = aEntrySize;
     PL_INIT_ARENA_POOL(&mWordPool, "Words Arena", 16384);
     bool ok = PL_DHashTableInit(&mTokenTable, &gTokenTableOps, nullptr,
-                                  aEntrySize, 256);
+                                aEntrySize, 256, mozilla::fallible_t());
     NS_ASSERTION(ok, "mTokenTable failed to initialize");
     if (!ok)
       PR_LOG(BayesianFilterLogModule, PR_LOG_ERROR, ("mTokenTable failed to initialize"));
@@ -208,7 +208,7 @@ nsresult TokenHash::clearTokens()
         PL_DHashTableFinish(&mTokenTable);
         PL_FreeArenaPool(&mWordPool);
         ok = PL_DHashTableInit(&mTokenTable, &gTokenTableOps, nullptr,
-                               mEntrySize, 256);
+                               mEntrySize, 256, mozilla::fallible_t());
         NS_ASSERTION(ok, "mTokenTable failed to initialize");
         if (!ok)
           PR_LOG(BayesianFilterLogModule, PR_LOG_ERROR, ("mTokenTable failed to initialize in clearTokens()"));
@@ -985,7 +985,7 @@ TokenStreamListener::~TokenStreamListener()
     delete mAnalyzer;
 }
 
-NS_IMPL_ISUPPORTS3(TokenStreamListener, nsIRequestObserver, nsIStreamListener, nsIMsgHeaderSink)
+NS_IMPL_ISUPPORTS(TokenStreamListener, nsIRequestObserver, nsIStreamListener, nsIMsgHeaderSink)
 
 NS_IMETHODIMP TokenStreamListener::ProcessHeaders(nsIUTF8StringEnumerator *aHeaderNames, nsIUTF8StringEnumerator *aHeaderValues, bool dontCollectAddress)
 {
@@ -1015,7 +1015,8 @@ NS_IMETHODIMP TokenStreamListener::OnEndMsgDownload(nsIMsgMailNewsUrl *url)
 }
 
 
-NS_IMETHODIMP TokenStreamListener::OnMsgHasRemoteContent(nsIMsgDBHdr * aMsgHdr)
+NS_IMETHODIMP TokenStreamListener::OnMsgHasRemoteContent(nsIMsgDBHdr *aMsgHdr,
+                                                         nsIURI *aContentURI)
 {
     return NS_OK;
 }
@@ -1169,7 +1170,7 @@ NS_IMETHODIMP TokenStreamListener::OnStopRequest(nsIRequest *aRequest, nsISuppor
 
 /* Implementation file */
 
-NS_IMPL_ISUPPORTS5(nsBayesianFilter, nsIMsgFilterPlugin,
+NS_IMPL_ISUPPORTS(nsBayesianFilter, nsIMsgFilterPlugin,
                    nsIJunkMailPlugin, nsIMsgCorpus, nsISupportsWeakReference,
                    nsIObserver)
 
@@ -2718,7 +2719,7 @@ void CorpusStore::remove(const char* word, uint32_t aTraitId, uint32_t aCount)
 
 uint32_t CorpusStore::getMessageCount(uint32_t aTraitId)
 {
-  uint32_t index = mMessageCountsId.IndexOf(aTraitId);
+  size_t index = mMessageCountsId.IndexOf(aTraitId);
   if (index == mMessageCountsId.NoIndex)
     return 0;
   return mMessageCounts.ElementAt(index);
@@ -2726,7 +2727,7 @@ uint32_t CorpusStore::getMessageCount(uint32_t aTraitId)
 
 void CorpusStore::setMessageCount(uint32_t aTraitId, uint32_t aCount)
 {
-  uint32_t index = mMessageCountsId.IndexOf(aTraitId);
+  size_t index = mMessageCountsId.IndexOf(aTraitId);
   if (index == mMessageCountsId.NoIndex)
   {
     mMessageCounts.AppendElement(aCount);
