@@ -63,7 +63,7 @@ static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
 /* the following macros actually implement addref, release and query interface for our component. */
-NS_IMPL_ISUPPORTS_INHERITED2(nsMsgMailboxParser,
+NS_IMPL_ISUPPORTS_INHERITED(nsMsgMailboxParser,
                              nsParseMailMessageState,
                              nsIStreamListener,
                              nsIRequestObserver)
@@ -291,7 +291,7 @@ void nsMsgMailboxParser::UpdateStatusText (const char* stringName)
     if (NS_FAILED(rv))
       return;
     nsString finalString;
-    const PRUnichar * stringArray[] = { m_folderName.get() };
+    const char16_t * stringArray[] = { m_folderName.get() };
     rv = bundle->FormatStringFromName(NS_ConvertASCIItoUTF16(stringName).get(),
                                       stringArray, 1, getter_Copies(finalString));
     m_statusFeedback->ShowStatusString(finalString);
@@ -507,7 +507,7 @@ nsMsgMailboxParser::AcquireFolderLock()
   return folder->AcquireSemaphore(supports);
 }
 
-NS_IMPL_ISUPPORTS2(nsParseMailMessageState, nsIMsgParseMailMsgState, nsIDBChangeListener)
+NS_IMPL_ISUPPORTS(nsParseMailMessageState, nsIMsgParseMailMsgState, nsIDBChangeListener)
 
 nsParseMailMessageState::nsParseMailMessageState()
 {
@@ -553,7 +553,6 @@ nsParseMailMessageState::nsParseMailMessageState()
      }
   }
   Clear();
-  m_HeaderAddressParser = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
 }
 
 nsParseMailMessageState::~nsParseMailMessageState()
@@ -929,9 +928,11 @@ nsresult nsParseMailMessageState::ParseHeaders ()
   char *buf = m_headers.GetBuffer();
   uint32_t buf_length = m_headers.GetBufferPos();
   char *buf_end = buf + buf_length;
+  MOZ_ASSERT(buf_length > 1 && (buf[buf_length - 1] == '\r' ||
+    buf[buf_length - 1] == '\n'), "Header text should always end in a newline");
   while (buf < buf_end)
   {
-    char *colon = PL_strnchr(buf, ':', buf_length);
+    char *colon = PL_strnchr(buf, ':', buf_end - buf);
     char *end;
     char *value = 0;
     struct message_header *header = 0;
@@ -1044,7 +1045,7 @@ nsresult nsParseMailMessageState::ParseHeaders ()
 #endif
 
       ToLowerCase(headerStr);
-      uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
+      size_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
       if (customHeaderIndex != m_customDBHeaders.NoIndex)
         header = & m_customDBHeaderValues[customHeaderIndex];
     }
@@ -1055,7 +1056,7 @@ nsresult nsParseMailMessageState::ParseHeaders ()
 SEARCH_NEWLINE:
     // move past any non terminating characters, rewriting them if folding white space
     // exists
-    while (buf <= buf_end && *buf != '\r' && *buf != '\n')
+    while (buf < buf_end && *buf != '\r' && *buf != '\n')
     {
       if (writeOffset)
         *(buf - writeOffset) = *buf;
@@ -1088,6 +1089,17 @@ SEARCH_NEWLINE:
         buf++;
         writeOffset++;
       }
+
+      // If we get here, the message headers ended in an empty line, like:
+      // To: blah blah blah<CR><LF>  <CR><LF>[end of buffer]. The code below
+      // requires buf to land on a newline to properly null-terminate the
+      // string, so back up a tad so that it is pointing to one.
+      if (buf == buf_end)
+      {
+        --buf;
+        MOZ_ASSERT(*buf == '\n' || *buf == '\r',
+          "Header text should always end in a newline.");
+      }
       goto SEARCH_NEWLINE;
     }
 
@@ -1095,7 +1107,7 @@ SEARCH_NEWLINE:
     {
       value = colon + 1;
       // eliminate trailing blanks after the colon
-      while (*value == ' ' || *value == '\t')
+      while (value < (buf - writeOffset) && (*value == ' ' || *value == '\t'))
         value++;
 
       header->value = value;
@@ -1107,7 +1119,7 @@ SEARCH_NEWLINE:
     {
       char *last = buf - writeOffset;
       char *saveBuf = buf;
-      if (*buf == '\r' && buf[1] == '\n')
+      if (*buf == '\r' && buf + 1 < buf_end && buf[1] == '\n')
         buf++;
       buf++;
       // null terminate the left-over slop so we don't confuse msg filters.
@@ -1154,6 +1166,9 @@ SEARCH_NEWLINE:
           }
         }
       }
+
+      MOZ_ASSERT(header->value[header->length] == 0,
+        "Non-null-terminated strings cause very, very bad problems");
     }
   }
   return NS_OK;
@@ -1280,7 +1295,6 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
   struct message_header *content_type;
   char md5_data [50];
 
-  const char *s;
   uint32_t flags = 0;
   uint32_t delta = 0;
   nsMsgPriorityValue priorityFlags = nsMsgPriority::notSet;
@@ -1328,11 +1342,8 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
   {
     if (mozstatus->length == 4)
     {
-      int i;
-      for (i=0,s=mozstatus->value ; i<4 ; i++,s++)
-      {
-        flags = (flags << 4) | msg_UnHex(*s);
-      }
+      NS_ASSERTION(MsgIsHex(mozstatus->value, 4), "Expected 4 hex digits for flags.");
+      flags = MsgUnhex(mozstatus->value, 4);
       // strip off and remember priority bits.
       flags &= ~nsMsgMessageFlags::RuntimeOnly;
       priorityFlags = (nsMsgPriorityValue) ((flags & nsMsgMessageFlags::Priorities) >> 13);
@@ -1452,61 +1463,16 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
       }
       else if (recipient)
       {
-        // note that we're now setting the whole recipient list,
-        // not just the pretty name of the first recipient.
-        uint32_t numAddresses;
-        char  *names;
-        char  *addresses;
-
-        ret = m_HeaderAddressParser->ParseHeaderAddresses(recipient->value,
-                                                          &names, &addresses,
-                                                          &numAddresses);
-        if (NS_SUCCEEDED(ret))
-        {
-          m_newMsgHdr->SetRecipientsArray(names, addresses, numAddresses);
-          PR_Free(addresses);
-          PR_Free(names);
-        }
-        else {  // hmm, should we just use the original string?
-          m_newMsgHdr->SetRecipients(recipient->value);
-        }
+        m_newMsgHdr->SetRecipients(recipient->value);
       }
       if (ccList)
       {
-        uint32_t numAddresses;
-        char  *names;
-        char  *addresses;
-
-        ret = m_HeaderAddressParser->ParseHeaderAddresses(ccList->value,
-                                                          &names, &addresses,
-                                                          &numAddresses);
-        if (NS_SUCCEEDED(ret) && numAddresses > 0)
-        {
-          m_newMsgHdr->SetCCListArray(names, addresses, numAddresses);
-          PR_Free(addresses);
-          PR_Free(names);
-        }
-        else  // hmm, should we just use the original string?
-          m_newMsgHdr->SetCcList(ccList->value);
+        m_newMsgHdr->SetCcList(ccList->value);
       }
 
       if (bccList)
       {
-        uint32_t numAddresses;
-        char  *names;
-        char  *addresses;
-
-        ret = m_HeaderAddressParser->ParseHeaderAddresses(bccList->value,
-                                                          &names, &addresses,
-                                                          &numAddresses);
-        if (NS_SUCCEEDED(ret))
-        {
-          m_newMsgHdr->SetBCCListArray(names, addresses, numAddresses);
-          PR_Free(addresses);
-          PR_Free(names);
-        }
-        else  // hmm, should we just use the original string?
-          m_newMsgHdr->SetBccList(bccList->value);
+        m_newMsgHdr->SetBccList(bccList->value);
       }
 
       rv = InternSubject (subject);
@@ -1541,7 +1507,7 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
         if (!mozstatus && statush)
         {
           /* Parse a little bit of the Berkeley Mail status header. */
-          for (s = statush->value; *s; s++) {
+          for (const char *s = statush->value; *s; s++) {
             uint32_t msgFlags = 0;
             (void)m_newMsgHdr->GetFlags(&msgFlags);
             switch (*s)
@@ -1720,7 +1686,7 @@ nsParseNewMailState::nsParseNewMailState()
   m_numNotNewMessages = 0;
  }
 
-NS_IMPL_ISUPPORTS_INHERITED1(nsParseNewMailState, nsMsgMailboxParser, nsIMsgFilterHitNotify)
+NS_IMPL_ISUPPORTS_INHERITED(nsParseNewMailState, nsMsgMailboxParser, nsIMsgFilterHitNotify)
 
 nsresult
 nsParseNewMailState::Init(nsIMsgFolder *serverFolder, nsIMsgFolder *downloadFolder,
@@ -2490,14 +2456,18 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     return NS_MSG_NOT_A_MAIL_FOLDER;
   }
 
-  nsCOMPtr <nsIMsgLocalMailFolder> destLocalFolder = do_QueryInterface(destIFolder);
-  if (destLocalFolder)
-  {
-    bool destFolderTooBig;
-    destLocalFolder->WarnIfLocalFileTooBig(msgWindow, &destFolderTooBig);
-    if (destFolderTooBig)
+  uint32_t messageLength;
+  mailHdr->GetMessageSize(&messageLength);
+
+  nsCOMPtr<nsIMsgLocalMailFolder> localFolder = do_QueryInterface(destIFolder);
+  if (localFolder) {
+    bool destFolderTooBig = true;
+    rv = localFolder->WarnIfLocalFileTooBig(msgWindow, messageLength,
+                                            &destFolderTooBig);
+    if (NS_FAILED(rv) || destFolderTooBig)
       return NS_MSG_ERROR_WRITING_MAIL_FOLDER;
   }
+
   nsCOMPtr<nsISupports> myISupports =
     do_QueryInterface(static_cast<nsIMsgParseMailMsgState*>(this));
 
@@ -2517,7 +2487,6 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     return NS_MSG_FOLDER_UNREADABLE;  // ### dmb
   }
 
-  nsCOMPtr<nsIMsgLocalMailFolder> localFolder = do_QueryInterface(destIFolder);
   nsCOMPtr<nsIMsgDatabase> destMailDB;
 
   if (!localFolder)
@@ -2535,12 +2504,14 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
                                             getter_AddRefs(newHdr));
   if (NS_SUCCEEDED(rv) && !newHdr)
     rv = NS_ERROR_UNEXPECTED;
-  if (NS_SUCCEEDED(rv))
+
+  if (NS_FAILED(rv))
   {
-    uint32_t messageLength;
-    mailHdr->GetMessageSize(&messageLength);
-    rv = AppendMsgFromStream(inputStream, newHdr, messageLength,
-                             destIFolder);
+    destIFolder->ThrowAlertMsg("filterFolderHdrAddFailed", msgWindow);
+  } else {
+    rv = AppendMsgFromStream(inputStream, newHdr, messageLength, destIFolder);
+    if (NS_FAILED(rv))
+      destIFolder->ThrowAlertMsg("filterFolderWriteFailed", msgWindow);
   }
 
   if (NS_FAILED(rv))
@@ -2548,11 +2519,8 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     if (destMailDB)
       destMailDB->Close(true);
 
-    if (destIFolder)
-    {
-      destIFolder->ReleaseSemaphore(myISupports);
-      destIFolder->ThrowAlertMsg("filterFolderWriteFailed", msgWindow);
-    }
+    destIFolder->ReleaseSemaphore(myISupports);
+
     return NS_MSG_ERROR_WRITING_MAIL_FOLDER;
   }
 
