@@ -1,43 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Seth Spitzer <sspitzer@netscape.com>
- *   Håkan Waara  <hwaara@chello.se>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Markus Hossner <markushossner@gmx.de>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
@@ -109,59 +73,11 @@ static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
 #define kNewsSortOffset 9000
 
+#define kSizeUnknown 1
+
 #define NEWS_SCHEME "news:"
 #define SNEWS_SCHEME "snews:"
 
-////////////////////////////////////////////////////////////////////////////////
-
-namespace {
-class AsyncAuthMigrator : public nsIMsgAsyncPromptListener {
-public:
-  AsyncAuthMigrator(nsIMsgNewsFolder *folder) : m_folder(folder) {}
-
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIMSGASYNCPROMPTLISTENER
-
-  void EnqueuePrompt();
-
-private:
-  nsCOMPtr<nsIMsgNewsFolder> m_folder;
-};
-
-NS_IMPL_ISUPPORTS1(AsyncAuthMigrator, nsIMsgAsyncPromptListener)
-
-NS_IMETHODIMP AsyncAuthMigrator::OnPromptStart(bool *retval)
-{
-  *retval = true;
-  return m_folder->MigrateLegacyCredentials();
-}
-
-NS_IMETHODIMP AsyncAuthMigrator::OnPromptAuthAvailable()
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP AsyncAuthMigrator::OnPromptCanceled()
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-void AsyncAuthMigrator::EnqueuePrompt()
-{
-  nsCOMPtr<nsIMsgAsyncPrompter> prompter =
-    do_GetService(NS_MSGASYNCPROMPTER_CONTRACTID);
-
-  // Make up a fake unique key to prevent coalescing of prompts
-  // The address of this object should be sufficient
-  nsAutoCString queueKey;
-  queueKey.AppendInt((int32_t)(uint64_t)this);
-  prompter->QueueAsyncAuthPrompt(queueKey, false, this);
-}
-
-}
-
- 
-
 ////////////////////////////////////////////////////////////////////////////////
 
 nsMsgNewsFolder::nsMsgNewsFolder(void) :
@@ -171,6 +87,7 @@ nsMsgNewsFolder::nsMsgNewsFolder(void) :
      mReadSet(nullptr), mSortOrder(kNewsSortOffset)
 {
   MOZ_COUNT_CTOR(nsNewsFolder); // double count these for now.
+  mFolderSize = kSizeUnknown;
 }
 
 nsMsgNewsFolder::~nsMsgNewsFolder(void)
@@ -268,10 +185,6 @@ nsMsgNewsFolder::AddNewsgroup(const nsACString &name, const nsACString& setStr,
   // cache this for when we open the db
   rv = newsFolder->SetReadSetFromStr(setStr);
 
-  // I don't have a good time to do this, but this is as good as any...
-  nsRefPtr<AsyncAuthMigrator> delayedPrompt(new AsyncAuthMigrator(newsFolder));
-  delayedPrompt->EnqueuePrompt();
-
   rv = folder->SetParent(this);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -426,6 +339,7 @@ nsMsgNewsFolder::UpdateFolder(nsIMsgWindow *aWindow)
   // We're not getting messages because either get_messages_on_select is
   // false or we're offline. Send an immediate folder loaded notification.
   NotifyFolderEvent(mFolderLoadedAtom);
+  (void) RefreshSizeOnDisk();
   return NS_OK;
 }
 
@@ -480,22 +394,6 @@ nsMsgNewsFolder::GetCanCompact(bool *aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgNewsFolder::GetCanDeleteMessages(bool *aResult)
-{
-  NS_ENSURE_ARG_POINTER(aResult);
-
-  nsresult rv;
-  nsCOMPtr<nsIPrefBranch> prefBranch =
-    do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Don't allow delete by default
-  *aResult = false;
-  prefBranch->GetBoolPref("news.allow_delete_with_no_undo", aResult);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsMsgNewsFolder::GetMessages(nsISimpleEnumerator **result)
 {
@@ -564,7 +462,7 @@ NS_IMETHODIMP nsMsgNewsFolder::CreateSubfolder(const nsAString& newsgroupName,
 {
   nsresult rv = NS_OK;
   if (newsgroupName.IsEmpty())
-    return NS_ERROR_FAILURE;
+    return NS_MSG_ERROR_INVALID_FOLDER_NAME;
 
   nsCOMPtr<nsIMsgFolder> child;
   // Create an empty database for this mail folder, set its name from the user
@@ -644,6 +542,8 @@ NS_IMETHODIMP nsMsgNewsFolder::Delete()
   rv = nntpServer->RemoveNewsgroup(name);
   NS_ENSURE_SUCCESS(rv,rv);
 
+  (void) RefreshSizeOnDisk();
+
   return SetNewsrcHasChanged(true);
 }
 
@@ -726,7 +626,7 @@ nsresult nsMsgNewsFolder::AbbreviatePrettyName(nsAString& prettyName, int32_t fu
   int32_t newword = 0;     // == 2 if done with all abbreviated words
 
   fullwords = 0;
-  PRUnichar currentChar;
+  char16_t currentChar;
   for (int32_t i = 1; i < length; i++)
   {
     // this temporary assignment is needed to fix an intel mac compiler bug.
@@ -839,13 +739,57 @@ NS_IMETHODIMP nsMsgNewsFolder::GetExpungedBytesCount(uint32_t *count)
 
 NS_IMETHODIMP nsMsgNewsFolder::GetDeletable(bool *deletable)
 {
+  NS_ENSURE_ARG_POINTER(deletable);
+
   *deletable = false;
+  // For legacy reasons, there can be Saved search folders under news accounts.
+  // Allow deleting those.
+  GetFlag(nsMsgFolderFlags::Virtual, deletable);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgNewsFolder::RefreshSizeOnDisk()
+{
+  uint64_t oldFolderSize = mFolderSize;
+  // We set size to unknown to force it to get recalculated from disk.
+  mFolderSize = kSizeUnknown;
+  if (NS_SUCCEEDED(GetSizeOnDisk(&mFolderSize)))
+    NotifyIntPropertyChanged(kFolderSizeAtom, oldFolderSize, mFolderSize);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::GetSizeOnDisk(uint32_t *size)
 {
-  return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_ARG_POINTER(size);
+
+  // 0 is a valid folder size (meaning empty file with no offline messages),
+  // but 1 is not. So use 1 as a special value meaning no file size was fetched
+  // from disk yet.
+  if (mFolderSize == kSizeUnknown)
+  {
+    nsCOMPtr<nsIFile> diskFile;
+    nsresult rv = GetFilePath(getter_AddRefs(diskFile));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // If there were no news messages downloaded for offline use, the folder file
+    // may not exist yet. In that case size is 0.
+    bool exists = false;
+    rv = diskFile->Exists(&exists);
+    if (NS_FAILED(rv) || !exists)
+    {
+      mFolderSize = 0;
+    }
+    else
+    {
+      int64_t fileSize;
+      rv = diskFile->GetFileSize(&fileSize);
+      NS_ENSURE_SUCCESS(rv, rv);
+      mFolderSize = fileSize;
+    }
+  }
+
+  *size = mFolderSize;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -889,6 +833,8 @@ nsMsgNewsFolder::DeleteMessages(nsIArray *messages, nsIMsgWindow *aMsgWindow,
     NotifyFolderEvent(NS_SUCCEEDED(rv) ? mDeleteOrMoveMsgCompletedAtom :
       mDeleteOrMoveMsgFailedAtom);
 
+  (void) RefreshSizeOnDisk();
+
   return NS_OK;
 }
 
@@ -1218,80 +1164,6 @@ nsresult nsMsgNewsFolder::CreateNewsgroupUrlForSignon(const char *ref,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgNewsFolder::MigrateLegacyCredentials()
-{
-  // The original ways that authentication credentials were stored was rather
-  // complicated and messy. We used separate URLs as the "HTTP realm" field to
-  // permit prompting for username and password as separate dialogs. In this
-  // method, we check for this, and store them in the new unified credentials
-  // dialog.
-
-  // Create the URLs that the login manager needs
-  nsString signonUrl;
-  nsresult rv = CreateNewsgroupUrlForSignon(nullptr, signonUrl);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsString usernameUrl;
-  rv = CreateNewsgroupUrlForSignon("username", usernameUrl);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsString passwordUrl;
-  rv = CreateNewsgroupUrlForSignon("password", passwordUrl);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsILoginManager> loginMgr =
-    do_GetService(NS_LOGINMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Grab out the saved username
-  uint32_t count = 0;
-  nsILoginInfo **logins = nullptr;
-  rv = loginMgr->FindLogins(&count, signonUrl, EmptyString(), usernameUrl,
-    &logins);
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(count <= 1, "Too many usernames?");
-
-  nsString username;
-  if (count > 0)
-  {
-    rv = logins[0]->GetPassword(username);
-    // Remove the saved login
-    loginMgr->RemoveLogin(logins[0]);
-  }
-
-  NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(count, logins);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Do the same things for the password
-  rv = loginMgr->FindLogins(&count, signonUrl, EmptyString(), passwordUrl,
-                            &logins);
-  NS_ENSURE_SUCCESS(rv, rv);
-  NS_ASSERTION(count <= 1, "Too many passwords?");
-
-  nsString password;
-  if (count > 0)
-  {
-    rv = logins[0]->GetPassword(password);
-    loginMgr->RemoveLogin(logins[0]);
-  }
-  NS_FREE_XPCOM_ISUPPORTS_POINTER_ARRAY(count, logins);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // If there is nothing to migrate, then do nothing
-  if (username.IsEmpty() && password.IsEmpty())
-    return NS_OK;
-
-  // Make and add the new logon
-  nsCOMPtr<nsILoginInfo> newLogin = do_CreateInstance(NS_LOGININFO_CONTRACTID);
-  // We need to pass in JS equivalent to "null"; empty ("") isn't good enough
-  nsString voidString;
-  voidString.SetIsVoid(true);
-  newLogin->Init(signonUrl, voidString, signonUrl, username, password,
-    EmptyString(), EmptyString());
-  return loginMgr->AddLogin(newLogin);
-}
-
 NS_IMETHODIMP
 nsMsgNewsFolder::GetAuthenticationCredentials(nsIMsgWindow *aMsgWindow,
     bool mayPrompt, bool mustPrompt, bool *validCredentials)
@@ -1364,7 +1236,7 @@ nsMsgNewsFolder::GetAuthenticationCredentials(nsIMsgWindow *aMsgWindow,
     {
       // Format the prompt text strings
       nsString promptTitle, promptText;
-      bundle->GetStringFromName(NS_LITERAL_STRING("enterUserPassTitle").get(),
+      bundle->GetStringFromName(MOZ_UTF16("enterUserPassTitle"),
         getter_Copies(promptTitle));
 
       nsString serverName;
@@ -1381,16 +1253,16 @@ nsMsgNewsFolder::GetAuthenticationCredentials(nsIMsgWindow *aMsgWindow,
       bool singleSignon = true;
       nntpServer->GetSingleSignon(&singleSignon);
 
-      const PRUnichar *params[2];
+      const char16_t *params[2];
       params[0] = mName.get();
       params[1] = serverName.get();
       if (singleSignon)
         bundle->FormatStringFromName(
-          NS_LITERAL_STRING("enterUserPassServer").get(),
+          MOZ_UTF16("enterUserPassServer"),
           &params[1], 1, getter_Copies(promptText));
       else
         bundle->FormatStringFromName(
-          NS_LITERAL_STRING("enterUserPassGroup").get(),
+          MOZ_UTF16("enterUserPassGroup"),
           params, 2, getter_Copies(promptText));
 
       // Fill the signon url for the dialog
@@ -1399,9 +1271,9 @@ nsMsgNewsFolder::GetAuthenticationCredentials(nsIMsgWindow *aMsgWindow,
       NS_ENSURE_SUCCESS(rv, rv);
 
       // Prefill saved username/password
-      PRUnichar *uniGroupUsername = ToNewUnicode(
+      char16_t *uniGroupUsername = ToNewUnicode(
         NS_ConvertASCIItoUTF16(mGroupUsername));
-      PRUnichar *uniGroupPassword = ToNewUnicode(
+      char16_t *uniGroupPassword = ToNewUnicode(
         NS_ConvertASCIItoUTF16(mGroupPassword));
 
       // Prompt for the dialog
@@ -1723,7 +1595,7 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadAllForOffline(nsIUrlListener *listener, n
 {
   nsTArray<nsMsgKey> srcKeyArray;
   SetSaveArticleOffline(true);
-  nsresult rv;
+  nsresult rv = NS_OK;
 
   // build up message keys.
   if (mDatabase)
@@ -1735,9 +1607,10 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadAllForOffline(nsIUrlListener *listener, n
       bool hasMore;
       while (NS_SUCCEEDED(rv = enumerator->HasMoreElements(&hasMore)) && hasMore)
       {
-        nsCOMPtr <nsIMsgDBHdr> pHeader;
-        rv = enumerator->GetNext(getter_AddRefs(pHeader));
+        nsCOMPtr <nsISupports> supports;
+        rv = enumerator->GetNext(getter_AddRefs(supports));
         NS_ASSERTION(NS_SUCCEEDED(rv), "nsMsgDBEnumerator broken");
+        nsCOMPtr <nsIMsgDBHdr> pHeader = do_QueryInterface(supports);
         if (pHeader && NS_SUCCEEDED(rv))
         {
           bool shouldStoreMsgOffline = false;
@@ -1754,7 +1627,9 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadAllForOffline(nsIUrlListener *listener, n
   if (!downloadState)
     return NS_ERROR_OUT_OF_MEMORY;
   m_downloadingMultipleMessages = true;
-  return downloadState->DownloadArticles(msgWindow, this, &srcKeyArray);
+  rv = downloadState->DownloadArticles(msgWindow, this, &srcKeyArray);
+  (void) RefreshSizeOnDisk();
+  return rv;
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::DownloadMessagesForOffline(nsIArray *messages, nsIMsgWindow *window)
@@ -1780,7 +1655,10 @@ NS_IMETHODIMP nsMsgNewsFolder::DownloadMessagesForOffline(nsIArray *messages, ns
   if (!downloadState)
     return NS_ERROR_OUT_OF_MEMORY;
   m_downloadingMultipleMessages = true;
-  return downloadState->DownloadArticles(window, this, &srcKeyArray);
+
+  rv = downloadState->DownloadArticles(window, this, &srcKeyArray);
+  (void) RefreshSizeOnDisk();
+  return rv;
 }
 
 // line does not have a line terminator (e.g., CR or CRLF)
@@ -1852,6 +1730,7 @@ NS_IMETHODIMP nsMsgNewsFolder::Compact(nsIUrlListener *aListener, nsIMsgWindow *
   rv = GetDatabase();
   if (mDatabase)
     ApplyRetentionSettings();
+  (void) RefreshSizeOnDisk();
   return rv;
 }
 
